import sys

import pygame
import pygame.gfxdraw
import pygame_gui

sys.path.insert(0, '../frontiersman')


class TitleScreen:

    def __init__(self, manager, offer, cost, callback1, callback2):
        self.children = []
        STARTING_RESOLUTION = "test"
        RESOLUTION_DICT = {
            "1080p": (1920, 1080),
            "720p": (1280, 720),
            "test": (600, 400)
        }

        self.offer = (0, 0, 0, 0, 0)
        self.cost = (0, 0, 0, 0, 0)
        self.text_color = (255, 255, 255)
        self.manager = manager

        self.resolution = RESOLUTION_DICT.get(STARTING_RESOLUTION)
        self.menu_buttons = []
        pygame.init()

        self.screen_size = RESOLUTION_DICT.get(STARTING_RESOLUTION)
        pygame.display.set_caption('Frontiersman Menu')

        self.panel_art = []
        self.panel_art_scaled = [None, None, None, None, None, None]
        self.init_textures()

        self.window = self.init_window(self.screen_size)
        self.manager = self.init_manager(self.screen_size)

        self.graphics = self.generate_graphics_options()

        self.state = 'main'  # main, options, join

        running = True
        clock = pygame.time.Clock()

        while running:
            time_delta = clock.tick(60) / 1000.0
            for event in pygame.event.get():
                # close if X is clicked
                if event.type == pygame.QUIT:
                    running = False
                # close if ESC is pressed
                if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                    running = False

                if event.type == pygame.USEREVENT:
                    if self.state == 'main':
                        if event.user_type == pygame_gui.UI_BUTTON_PRESSED:
                            if event.ui_element == self.menu_buttons[0]:
                                callback1()
                            if event.ui_element == self.menu_buttons[1]:
                                callback2()

                self.manager.process_events(event)

            self.manager.update(time_delta)
            self.window.blit(self.graphics, (0, 0))
            self.manager.draw_ui(self.window)
            pygame.display.update()

        pygame.quit()

    @staticmethod
    def init_window(size):
        return pygame.display.set_mode(size)

    @staticmethod
    def init_manager(size):
        return pygame_gui.UIManager(size, '../themes/main_menu.json')

    def init_textures(self):
        # brick, ore, sheep, wheat, wood
        self.panel_art.append(pygame.image.load('../assets/cards/res/bricks.png'))
        self.panel_art.append(pygame.image.load('../assets/cards/res/ore.png'))
        self.panel_art.append(pygame.image.load('../assets/cards/res/sheep.png'))
        self.panel_art.append(pygame.image.load('../assets/cards/res/wheat.png'))
        self.panel_art.append(pygame.image.load('../assets/cards/res/wood.png'))

    def generate_graphics(self):
        self.generate_graphics_options()

    def generate_graphics_options(self):
        # define center coordinates
        center = (self.screen_size[0] / 2, self.screen_size[1] / 2)

        # create surface and apply background color
        background_color = (61, 120, 180)
        background = pygame.Surface(self.screen_size)
        background.fill(background_color)

        self.manager = TitleScreen.init_manager(self.screen_size)

        # button dimensions
        button_width = self.screen_size[0] // 10 * 2
        button_height = self.screen_size[1] // 10

        self.menu_buttons = []

        # function to add menu buttons to UI manager
        def menu_button(location, text):
            self.menu_buttons.append(pygame_gui.elements.UIButton(
                relative_rect=pygame.Rect(location, (button_width, button_height)),
                text=text, manager=self.manager,
                object_id='#menu_button_square_back'))

        padding = button_width // 2 + 10

        menu_button((center[0] - button_width // 2 - padding, int(9 * self.screen_size[1] / 10) - button_height // 2),
                    'Accept')
        menu_button((center[0] - button_width // 2 + padding, int(9 * self.screen_size[1] / 10) - button_height // 2),
                    'Deny')

        pygame.font.init()
        label_font = pygame.font.Font("../data/fonts/impact.ttf", self.screen_size[1] // 20)
        label_surface = pygame.Surface(self.screen_size, pygame.SRCALPHA)

        offer_text = pygame.font.Font.render(pygame.font.Font(
            "../data/fonts/impact.ttf",
            self.screen_size[1] // 15),
            "Offering", True, (255, 255, 255), None)

        if offer_text.get_width() > self.screen_size[0]:
            scale = self.screen_size[0] / offer_text.get_width()
            title_text = pygame.transform.scale(offer_text,
                                                (int(offer_text.get_width() * scale),
                                                 int(offer_text.get_height() * scale)))

        price_text = pygame.font.Font.render(pygame.font.Font(
            "../data/fonts/impact.ttf",
            self.screen_size[1] // 15),
            "For", True, (255, 255, 255), None)

        if price_text.get_width() > self.screen_size[0]:
            scale = self.screen_size[0] / price_text.get_width()
            price_text = pygame.transform.scale(price_text,
                                                (int(price_text.get_width() * scale),
                                                 int(price_text.get_height() * scale)))

        card_width = self.screen_size[0] // 12
        card_height = int(card_width * 7 / 5)
        # scale cards
        index = 0
        for textures in self.panel_art:
            self.panel_art_scaled[index] = pygame.transform.scale(textures, (card_width, card_height))
            index += 1

        index = 0
        h_padding = self.screen_size[0] // 20
        v_padding = self.screen_size[1] // 20
        card_location = (h_padding * 6, v_padding * 4)

        amount_font = pygame.font.Font(
            "../data/fonts/impact.ttf",
            self.screen_size[1] // 20)

        # draw cards
        for index in range(0, 5):
            background.blit(self.panel_art_scaled[index], (
                int(card_location[0] + index * h_padding * 2 - self.panel_art_scaled[index].get_width() / 2),
                int(card_location[1] - self.panel_art_scaled[index].get_height() / 2)))
            background.blit(self.panel_art_scaled[index], (
                int(card_location[0] + index * h_padding * 2 - self.panel_art_scaled[index].get_width() / 2),
                int(card_location[1] - self.panel_art_scaled[index].get_height() / 2 + 8 * v_padding)))

        # draw amounts
        index = 0
        for offer in self.offer:
            text = pygame.font.Font.render(amount_font, str(offer), True, self.text_color, None)
            background.blit(text, (int(card_location[0] + index * h_padding * 2 - text.get_width() / 2),
                            int(card_location[1] - text.get_height() / 2 + v_padding * 3)))
            index += 1

        index = 0
        for offer in self.cost:
            text = pygame.font.Font.render(amount_font, str(offer), True, self.text_color, None)
            background.blit(text, (int(card_location[0] + index * h_padding * 2 - text.get_width() / 2),
                                   int(card_location[1] - text.get_height() / 2 + v_padding * 11)))
            index += 1

        # create screen sized software surface
        title_surface = pygame.Surface(self.screen_size, pygame.SRCALPHA)

        # apply title to screen sized software surface
        title_surface.blit(offer_text,
                           (int(self.screen_size[0] / 2 - offer_text.get_width() / 2),
                            int(self.screen_size[1] / 20 - offer_text.get_height() / 2)))
        title_surface.blit(price_text,
                           (int(self.screen_size[0] / 2 - price_text.get_width() / 2),
                            int(self.screen_size[1] / 20 * 9 - price_text.get_height() / 2)))

        # apply title to background surface
        background.blit(title_surface, (0, 0))
        # apply buttons to background surface
        background.blit(label_surface, (0, 0))

        return background


if __name__ == "__main__":
    TitleScreen()



import pygame
from typing import Union
from pygame_gui.core import UIElement, IContainerLikeInterface
from pygame_gui.elements import UIWindow, UIImage, UIVerticalScrollBar


# todo resize properly

class Notification:
    def __init__(self, ui_manager, size, text, array=False):
        self.size = (size[0], size[1])
        self.background = pygame.Surface(self.size, flags=pygame.SRCALPHA)
        self.background.fill((255, 255, 255))
        self.manager = ui_manager
        self.background = self.background.convert()

        self.square = (size[0] - 32, size[1] - 59)

        self.text = []

        if not array:
            self.text.append(text)
        else:
            self.text = text

        self.text_surface = pygame.Surface(self.square, flags=pygame.SRCALPHA)

        pygame.font.init()
        self.font = pygame.font.Font("./data/fonts/impact.ttf", 24)
        self.text_height = 24
        self.line_padding = 24

        num_lines = len(self.text)
        center_x = self.square[0] // 2
        center_y = self.square[1] // 2

        if num_lines % 2 == 0:
            start_y = int(center_y - (num_lines // 2 + 1) * self.text_height - (num_lines // 2 - 0.5 * self.line_padding))
        else:
            start_y = int(center_y - ((num_lines // 2) + 0.5) * self.text_height - (num_lines // 2 * self.line_padding))
        for index in range(0, num_lines):
            text_line = pygame.font.Font.render(self.font, self.text[index], True, (0, 0, 0), None)
            self.text_surface.blit(
                text_line,
                (int(center_x - text_line.get_width() // 2),
                 int(start_y + index * (self.text_height + self.line_padding))))

        self.background.blit(
            self.text_surface, (
                int(self.square[0] // 2 - self.text_surface.get_width() // 2),
                int(self.square[1] // 2 - self.text_surface.get_height() // 2))
        )

    def draw(self, surface):
        surface.blit(self.background, (0, 0))


class PopUp(UIWindow):
    def __init__(self, position, ui_manager, text, array=False, size=(300, 100)):
        self.size = size
        super().__init__(pygame.Rect(position, self.size), ui_manager,
                         window_display_title='Notification',
                         object_id='#help_window')

        surface_size = self.get_container().get_size()
        self.surface_element = UIImage(pygame.Rect((0, 0),
                                                   surface_size),
                                       pygame.Surface(surface_size).convert(),
                                       manager=ui_manager,
                                       container=self,
                                       parent_element=self)

        self.text_image = Notification(ui_manager, self.size, text, array)

        self.is_active = False

        self.set_blocking(True)

    def update(self, time_delta):
        super().update(time_delta)

        self.text_image.draw(self.surface_element.image)



import pygame.gfxdraw
import pygame_gui
import pygame_gui as pg_g
from pygame_gui.core import UIElement, IContainerLikeInterface
from pygame_gui.elements import UIWindow, UIImage, UIVerticalScrollBar


class TradePrompt(UIWindow):
    def __init__(self, manager, offer, cost, hand, callback1, callback2, size=(1000, 800), location=(100, 100)):
        print(offer)
        self.size = size
        self.offer = [0, 0, 0, 0, 0]
        self.offer[0] = offer[0]
        self.offer[1] = offer[3]
        self.offer[2] = offer[4]
        self.offer[3] = offer[1]
        self.offer[4] = offer[2]
        self.functional_offer = offer
        self.cost = [0, 0, 0, 0, 0]
        self.cost[0] = cost[0]
        self.cost[1] = cost[3]
        self.cost[2] = cost[4]
        self.cost[3] = cost[1]
        self.cost[4] = cost[2]
        self.functional_cost = cost
        self.hand = hand
        super().__init__(pygame.Rect(location, self.size), manager,
                         window_display_title='Trade Offer',
                         object_id='#help_window')

        surface_size = self.get_container().get_size()
        self.surface_element = UIImage(pygame.Rect((0, 0),
                                                   surface_size),
                                       pygame.Surface(surface_size).convert(),
                                       manager=manager,
                                       container=self,
                                       parent_element=self)

        self.text_color = (255, 255, 255)
        self.manager = manager
        self.menu_buttons = []

        self.callback1 = callback1
        self.callback2 = callback2

        self.screen_size = (size[0], size[1] - 50)
        pygame.display.set_caption('Frontiersman Menu')

        self.panel_art = []
        self.panel_art_scaled = [None, None, None, None, None, None]
        self.init_textures()

        self.graphics = self.generate_graphics()

        self.state = 'main'  # main, options, join

    def update(self, time_delta):
        super().update(time_delta)
        self.surface_element.image.blit(self.graphics, (0, 0))

    def handle_events(self, event):
        if event.type == pygame.USEREVENT:
            if event.user_type == pygame_gui.UI_BUTTON_PRESSED:
                if event.ui_element == self.menu_buttons[0]:
                    self.kill()
                    self.callback1()
                if event.ui_element == self.menu_buttons[1]:
                    self.kill()
                    self.callback2()
        self.manager.process_events(event)

    def process_event(self, event):
        if event.type == pygame.USEREVENT and event.user_type == 'ui_button_pressed' and event.ui_element == self.close_window_button:
            self.callback2()
        super().process_event(event)

    def init_textures(self):
        # brick, ore, sheep, wheat, wood
        self.panel_art.append(pygame.image.load('./assets/cards/res/bricks.png'))
        self.panel_art.append(pygame.image.load('./assets/cards/res/ore.png'))
        self.panel_art.append(pygame.image.load('./assets/cards/res/sheep.png'))
        self.panel_art.append(pygame.image.load('./assets/cards/res/wheat.png'))
        self.panel_art.append(pygame.image.load('./assets/cards/res/wood.png'))

    def generate_graphics(self):
        # define center coordinates
        center = (self.screen_size[0] / 2, self.screen_size[1] / 2)

        # create surface and apply background color
        background_color = (61, 120, 180)
        background = pygame.Surface(self.screen_size)
        background.fill(background_color)

        # button dimensions
        button_width = self.screen_size[0] // 10 * 2
        button_height = self.screen_size[1] // 10

        self.menu_buttons = []

        # function to add menu buttons to UI manager
        def menu_button(location, text):
            self.menu_buttons.append(pygame_gui.elements.UIButton(
                relative_rect=pygame.Rect(location, (button_width, button_height)),
                text=text, manager=self.manager,
                object_id='#menu_button_square_back', container=self))

        padding = button_width // 2 + 10

        menu_button((center[0] - button_width // 2 - padding, int(9 * self.screen_size[1] / 10) - button_height // 2),
                    'Accept')
        menu_button((center[0] - button_width // 2 + padding, int(9 * self.screen_size[1] / 10) - button_height // 2),
                    'Deny')
        for x in range(0, len(self.hand)):
            if (self.hand[x] - self.functional_cost[x] < 0):
                self.menu_buttons[0].disable()
                break

        pygame.font.init()
        label_font = pygame.font.Font("./data/fonts/impact.ttf", self.screen_size[1] // 20)
        label_surface = pygame.Surface(self.screen_size, pygame.SRCALPHA)

        offer_text = pygame.font.Font.render(pygame.font.Font(
            "./data/fonts/impact.ttf",
            self.screen_size[1] // 15),
            "Offering", True, (255, 255, 255), None)

        if offer_text.get_width() > self.screen_size[0]:
            scale = self.screen_size[0] / offer_text.get_width()
            title_text = pygame.transform.scale(offer_text,
                                                (int(offer_text.get_width() * scale),
                                                 int(offer_text.get_height() * scale)))

        price_text = pygame.font.Font.render(pygame.font.Font(
            "./data/fonts/impact.ttf",
            self.screen_size[1] // 15),
            "For", True, (255, 255, 255), None)

        if price_text.get_width() > self.screen_size[0]:
            scale = self.screen_size[0] / price_text.get_width()
            price_text = pygame.transform.scale(price_text,
                                                (int(price_text.get_width() * scale),
                                                 int(price_text.get_height() * scale)))

        card_width = self.screen_size[0] // 12
        card_height = int(card_width * 7 / 5)
        # scale cards
        index = 0
        for textures in self.panel_art:
            self.panel_art_scaled[index] = pygame.transform.scale(textures, (card_width, card_height))
            index += 1

        index = 0
        h_padding = self.screen_size[0] // 20
        v_padding = self.screen_size[1] // 20
        card_location = (h_padding * 6, v_padding * 4)

        amount_font = pygame.font.Font(
            "./data/fonts/impact.ttf",
            self.screen_size[1] // 20)

        # draw cards
        for index in range(0, 5):
            background.blit(self.panel_art_scaled[index], (
                int(card_location[0] + index * h_padding * 2 - self.panel_art_scaled[index].get_width() / 2),
                int(card_location[1] - self.panel_art_scaled[index].get_height() / 2)))
            background.blit(self.panel_art_scaled[index], (
                int(card_location[0] + index * h_padding * 2 - self.panel_art_scaled[index].get_width() / 2),
                int(card_location[1] - self.panel_art_scaled[index].get_height() / 2 + 8 * v_padding)))

        # draw amounts
        index = 0
        for offer in self.offer:
            text = pygame.font.Font.render(amount_font, str(offer), True, self.text_color, None)
            background.blit(text, (int(card_location[0] + index * h_padding * 2 - text.get_width() / 2),
                                   int(card_location[1] - text.get_height() / 2 + v_padding * 3)))
            index += 1

        index = 0
        for offer in self.cost:
            text = pygame.font.Font.render(amount_font, str(offer), True, self.text_color, None)
            background.blit(text, (int(card_location[0] + index * h_padding * 2 - text.get_width() / 2),
                                   int(card_location[1] - text.get_height() / 2 + v_padding * 11)))
            index += 1

        # create screen sized software surface
        title_surface = pygame.Surface(self.screen_size, pygame.SRCALPHA)

        # apply title to screen sized software surface
        title_surface.blit(offer_text,
                           (int(self.screen_size[0] / 2 - offer_text.get_width() / 2),
                            int(self.screen_size[1] / 20 - offer_text.get_height() / 2)))
        title_surface.blit(price_text,
                           (int(self.screen_size[0] / 2 - price_text.get_width() / 2),
                            int(self.screen_size[1] / 20 * 9 - price_text.get_height() / 2)))

        # apply title to background surface
        background.blit(title_surface, (0, 0))
        # apply buttons to background surface
        background.blit(label_surface, (0, 0))

        return background



import sys

import pygame
import pygame.gfxdraw
import pygame_gui
from pygame_gui.elements import UIDropDownMenu

sys.path.insert(0, '../frontiersman')


class TitleScreen:

    def __init__(self):
        self.children = []
        STARTING_RESOLUTION = "720p"
        RESOLUTION_DICT = {
            "1080p": (1920, 1080),
            "720p": (1280, 720)
        }

        self.resolution_translate_table = {
            "1920 x 1080": (1920, 1080),
            "1280 x 720": (1280, 720),
            "1600 x 1200": (1600, 1200),
            "800 x 600": (800, 600)
        }

        self.resolution = RESOLUTION_DICT.get(STARTING_RESOLUTION)
        self.menu_buttons = []
        pygame.init()

        self.screen_size = RESOLUTION_DICT.get(STARTING_RESOLUTION)
        pygame.display.set_caption('Frontiersman Menu')

        self.panel_circle_art = []
        self.panel_art_scaled = [None, None, None, None, None, None]
        self.panel_wood_texture = []
        self.init_textures()

        self.window = self.init_window(self.screen_size)
        self.manager = self.init_manager(self.screen_size)

        self.graphics = self.generate_graphics_main()
        # self.graphics = self.generate_graphics_options()

        self.state = 'main'  # main, options, join

        running = True
        clock = pygame.time.Clock()

        while running:
            time_delta = clock.tick(60) / 1000.0
            for event in pygame.event.get():
                # close if X is clicked
                if event.type == pygame.QUIT:
                    running = False
                if event.type == pygame.VIDEORESIZE:
                    self.resize()
                # close if ESC is pressed
                if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                    running = False

                if event.type == pygame.USEREVENT:
                    if self.state == 'main':
                        if event.user_type == pygame_gui.UI_BUTTON_PRESSED:
                            # action if client is pressed
                            if event.ui_element == self.menu_buttons[2]:
                                running = False
                            if event.ui_element == self.menu_buttons[0]:
                                self.state = 'game'
                                self.change_state()
                            if event.ui_element == self.menu_buttons[1]:
                                self.state = 'options'
                                self.change_state()
                    if self.state == 'options':
                        if (
                                event.user_type == pygame_gui.UI_DROP_DOWN_MENU_CHANGED and event.ui_element == self.resolution_drop_down):
                            self.resolution = self.resolution_translate_table.get(
                                self.resolution_drop_down.selected_option)
                        if event.user_type == pygame_gui.UI_BUTTON_PRESSED:
                            if event.ui_element == self.menu_buttons[0]:
                                self.state = 'main'
                                self.change_state()

                self.manager.process_events(event)

            self.manager.update(time_delta)
            self.window.blit(self.graphics, (0, 0))
            self.manager.draw_ui(self.window)
            pygame.display.update()

        pygame.quit()

    @staticmethod
    def init_window(size):
        return pygame.display.set_mode(size, flags=pygame.RESIZABLE)

    @staticmethod
    def init_manager(size):
        return pygame_gui.UIManager(size, '../themes/main_menu.json')

    def init_textures(self):
        #brick, ore, sheep, wheat, wood
        self.panel_circle_art.append(pygame.image.load('../assets/cards/res/brick_circle.png'))
        self.panel_circle_art.append(pygame.image.load('../assets/cards/res/ore_circle.png'))
        self.panel_circle_art.append(pygame.image.load('../assets/cards/res/sheep_circle.png'))
        self.panel_circle_art.append(pygame.image.load('../assets/cards/res/wheat_circle.png'))
        self.panel_circle_art.append(pygame.image.load('../assets/cards/res/wood_circle.png'))
        self.panel_circle_art.append(pygame.image.load('../assets/cards/res/card_back_square.png'))

        self.panel_wood_texture.append(pygame.image.load('../assets/main/wood_texture.png'))


    def change_state(self):
        self.graphics = self.generate_graphics()

    def generate_graphics(self):
        if self.state == 'options':
            return self.generate_graphics_options()

        if self.state == 'main':
            return self.generate_graphics_main()

        if self.state == 'options':
            pass
            # return self.generate_graphics_join()

        if self.state == 'game':
            return self.generate_graphics_gameui()

    def generate_graphics_options(self):
        # define center coordinates
        center = (self.screen_size[0] / 2, self.screen_size[1] / 2)

        # create surface and apply background color
        background_color = (61, 120, 180)
        background = pygame.Surface(self.screen_size)
        background.fill(background_color)

        self.manager = TitleScreen.init_manager(self.screen_size)

        button_labels = [
            ('Resolution:', ['1920 x 1080',
                             '1280 x 720',
                             '800 x 600',
                             '1600 x 1200']),
            ('Fullscreen:', ['True', 'False']),
            ('Borderless:', ['True', 'False'])
        ]

        # button dimensions
        button_width = self.screen_size[0] // 10 * 5
        button_height = self.screen_size[1] // 10

        self.menu_buttons = []

        # function to add menu buttons to UI manager
        def menu_button(location, text):
            self.menu_buttons.append(pygame_gui.elements.UIButton(
                relative_rect=pygame.Rect(location, (button_width, button_height)),
                text=text, manager=self.manager,
                object_id='#menu_button_square_back'))

        # create back button
        menu_button((center[0] - button_width // 2, int(8 * self.screen_size[1] / 10) - button_height // 2), 'Back')

        pygame.font.init()
        label_font = pygame.font.Font("../data/fonts/impact.ttf", self.screen_size[1] // 20)
        label_surface = pygame.Surface(self.screen_size, pygame.SRCALPHA)

        option_label_location = (center[0], self.screen_size[1] // (10 / 4))
        option_label_offset = self.screen_size[1] // 10

        padding = self.screen_size[0] // 40

        options_location = (center[0] + padding, self.screen_size[1] // (10 / 4))
        options_offset = self.screen_size[1] // 10
        options_size = (self.screen_size[0] // 5 - padding, self.screen_size[1] // 20)
        temp_size = 100

        # draw option labels
        for index in range(len(button_labels)):
            text = pygame.font.Font.render(label_font, button_labels[index][0], True, (255, 255, 255), None)
            temp_size = text.get_width()
            label_surface.blit(text, (option_label_location[0] - text.get_width(),
                                      option_label_offset * index + option_label_location[1] - text.get_height() // 2))

            # create placeholder
            if index > 0:
                text = pygame.font.Font.render(label_font, "True", True, (255, 255, 255), None)
                label_surface.blit(text, (option_label_location[0] + padding,
                                          option_label_offset * index + option_label_location[
                                              1] - text.get_height() // 2))

        options_size = (temp_size, self.screen_size[1] // 20)

        def get_resolution_string():
            return str(self.resolution[0]) + ' x ' + str(self.resolution[1])

        # 3 settings for font size, annoying but good enough

        if self.screen_size[1] > 600 and self.screen_size[0] > 950:
            size_id = '#drop_down_menu'

        elif self.screen_size[1] > 500 and self.screen_size[0] > 700:
            size_id = '#drop_down_menu_medium'
        else:
            size_id = '#drop_down_menu_small'

        # create drop down list for resolution options
        self.resolution_drop_down = UIDropDownMenu(button_labels[0][1],
                                                   get_resolution_string(),
                                                   pygame.Rect((int(options_location[0]),
                                                                int(options_location[1] - options_size[1] // 2)),
                                                               options_size),
                                                   self.manager,
                                                   object_id=size_id)

        # create title text
        title_text = pygame.font.Font.render(pygame.font.Font(
            "../data/fonts/impact.ttf",
            self.screen_size[1] // 5),
            "Options", True, (255, 255, 255), None)

        if title_text.get_width() > self.screen_size[0]:
            scale = self.screen_size[0] / title_text.get_width()
            title_text = pygame.transform.scale(title_text,
                                                (int(title_text.get_width() * scale),
                                                 int(title_text.get_height() * scale)))

        # create screen sized software surface
        title_surface = pygame.Surface(self.screen_size, pygame.SRCALPHA)

        # apply title to screen sized software surface
        title_surface.blit(title_text,
                           (int(self.screen_size[0] / 2 - title_text.get_width() / 2),
                            int(self.screen_size[1] / 5 - title_text.get_height() / 2)))

        # apply title to background surface
        background.blit(title_surface, (0, 0))
        # apply buttons to background surface
        background.blit(label_surface, (0, 0))

        return background

    def generate_graphics_gameui(self):
        # define center coordinates
        center = (self.screen_size[0] / 2, self.screen_size[1] / 2)

        # create surface and apply background color
        background_color = (61, 120, 180)
        background = pygame.Surface(self.screen_size)
        background.fill(background_color)

        self.manager = TitleScreen.init_manager(self.screen_size)

        def intify(location):
            return int(location[0]), int(location[1])

        def rounded_rectangle_flat_bottom(surface, location, width, height, radius, color):
            circle_centers = [intify((location[0] + radius,
                                      location[1] + radius)),
                              intify((location[0] + width - radius - 1,
                                      location[1] + radius))]
            for points in circle_centers:
                pygame.gfxdraw.filled_circle(surface, points[0], points[1], radius, color)

            pygame.gfxdraw.box(surface, pygame.Rect(
                intify((location[0] + radius, location[1])),
                intify((width - radius * 2, height))
            ), color)

            pygame.gfxdraw.box(surface, pygame.Rect(
                intify((location[0], location[1] + radius)),
                intify((width, height - radius))
            ), color)

        def rounded_rectangle(surface, location, width, height, radius, color):
            circle_x = [location[0] + radius, location[0] + width - radius - 1]
            circle_y = [location[1] + radius, location[1] + height - radius - 1]

            circle_centers = [
                (circle_x[0], circle_y[0]),
                (circle_x[0], circle_y[1]),
                (circle_x[1], circle_y[0]),
                (circle_x[1], circle_y[1])
            ]

            for points in circle_centers:
                pygame.gfxdraw.filled_circle(surface, points[0], points[1], radius, color)

            pygame.gfxdraw.box(surface, pygame.Rect(
                intify((location[0] + radius, location[1])),
                intify((width - radius * 2, height))
            ), color)

            pygame.gfxdraw.box(surface, pygame.Rect(
                intify((location[0], location[1] + radius)),
                intify((width, height - radius * 2))
            ), color)

        bottom_panel = pygame.Surface(self.screen_size, pygame.SRCALPHA)

        # bottom panel
        panel_width = self.screen_size[0] // 2
        panel_height = self.screen_size[1] // 10
        panel_corner_radius = panel_height // 2
        color = (255, 255, 255)

        rounded_rectangle_flat_bottom(bottom_panel,
                                     (center[0] - panel_width / 2, self.screen_size[1] - panel_height),
                                     panel_width, panel_height, panel_corner_radius, color)

        bottom_panel_mask = pygame.mask.from_surface(bottom_panel)
        #bottom_panel_mask.invert()
        wood_panel = pygame.Surface(self.screen_size, flags=pygame.SRCALPHA)
        for x in range(0, wood_panel.get_width() + 1):
            for y in range(0, wood_panel.get_height() + 1):
                wood_panel.blit(self.panel_wood_texture[0], (1000 * x, 1000 * y))

        bottom_panel_texture = bottom_panel_mask.to_surface(wood_panel, unsetcolor=None, unsetsurface=None)

        #resource circles
        circle_height = panel_height // 2
        circle_radius = int(panel_height // 3)
        circle_offset = (panel_width - 2 * panel_corner_radius) / 5
        circle_y = self.screen_size[1] - panel_height + panel_corner_radius
        circle_left = center[0] - panel_width / 2 + panel_corner_radius
        circle_centers = []

        for circle in range(0, 6):
            circle_centers.append(intify((circle_left + circle * circle_offset, circle_y)))

        index = 0
        for textures in self.panel_circle_art:
            self.panel_art_scaled[index] = pygame.transform.scale(textures, (circle_radius * 2, circle_radius * 2))
            index += 1

        index = 0
        for amount_panel in range(0, 5):
            rounded_rectangle(bottom_panel, (circle_centers[index][0], circle_centers[index][1]),
                              circle_radius * 2, circle_radius + 1, circle_radius // 8, (120, 120, 120))
            index += 1

        index = 0
        for center in circle_centers:
            bottom_panel.blit(self.panel_art_scaled[index], (center[0] - circle_radius, center[1] - circle_radius))
            index += 1
            pygame.gfxdraw.circle(bottom_panel, center[0], center[1], circle_radius, (0, 0, 0))

        background.blit(bottom_panel, (0, 0))
        #background.blit(bottom_panel_texture, (0, 0))

        return background

    def generate_graphics_main(self):

        # define center coordinates
        center = (self.screen_size[0] / 2, self.screen_size[1] / 2)

        # create surface and apply background color
        background_color = (61, 120, 180)
        background = pygame.Surface(self.screen_size)
        background.fill(background_color)

        self.manager = TitleScreen.init_manager(self.screen_size)

        # function to add menu buttons to UI manager
        def menu_button(location, text):
            self.menu_buttons.append(pygame_gui.elements.UIButton(
                relative_rect=pygame.Rect(location, (button_width, button_height)),
                text=text, manager=self.manager,
                object_id='#menu_button_square'))

        # the four buttons text
        button_labels = [
            'Join Game',
            'Options',
            'Exit'
        ]

        # button dimensions
        button_width = self.screen_size[0] // 10 * 5
        button_height = self.screen_size[1] // 10

        self.menu_buttons = []
        pygame.font.init()
        button_font = pygame.font.Font("../data/fonts/impact.ttf", self.screen_size[1] // 25)
        button_surface = pygame.Surface(self.screen_size, pygame.SRCALPHA)

        button_scale = 1
        thickness_0 = self.screen_size[0] // 300
        thickness_1 = self.screen_size[1] // 200
        thickness = max(min(thickness_0, thickness_1), 1)

        # add the four buttons
        for index in range(len(button_labels)):
            position = (int(center[0] - button_width / 2),
                        int((4 + index) * button_height))
            menu_button(position, "")

            # create background button text that works alongside py-game gui
            button_text = pygame.font.Font.render(button_font, button_labels[index], True, (255, 255, 255), None)

            if index == 0 and button_text.get_width() > button_width:
                button_scale = button_text.get_width() / button_width

            if button_scale != 1:
                button_text = pygame.transform.scale(button_text,
                                                     (int(button_text.get_width() / button_scale),
                                                      int(button_text.get_height() / button_scale)))

            button_surface.blit(
                button_text,
                (int(center[0] - button_text.get_width() / 2),
                 int((4.5 + index) * button_height - button_text.get_height() / 2))
            )

            # manually draw button borders
            '''
            for index in range(0, thickness):
                border = pygame.Rect((position[0] + index, position[1] + index),
                                     (button_width - 2 * index, button_height - 2 * index))
                pygame.draw.rect(button_surface, (255, 255, 255), border, width=1)
            '''

        # create title text
        title_text = pygame.font.Font.render(pygame.font.Font(
            "../data/fonts/impact.ttf",
            self.screen_size[1] // 5),
            "Frontiersman", True, (255, 255, 255), None)

        if title_text.get_width() > self.screen_size[0]:
            scale = self.screen_size[0] / title_text.get_width()
            title_text = pygame.transform.scale(title_text,
                                                (int(title_text.get_width() * scale),
                                                 int(title_text.get_height() * scale)))

        # create screen sized software surface
        title_surface = pygame.Surface(self.screen_size, pygame.SRCALPHA)

        # apply title to screen sized software surface
        title_surface.blit(title_text,
                           (int(self.screen_size[0] / 2 - title_text.get_width() / 2),
                            int(self.screen_size[1] / 5 - title_text.get_height() / 2)))

        # apply title to background surface
        background.blit(title_surface, (0, 0))
        # apply buttons to background surface
        background.blit(button_surface, (0, 0))

        return background

    def resize(self):
        WINDOW_WIDTH = pygame.display.get_window_size()[0]
        WINDOW_HEIGHT = pygame.display.get_window_size()[1]
        self.screen_size = (WINDOW_WIDTH, WINDOW_HEIGHT)
        self.graphics = self.generate_graphics()


if __name__ == "__main__":
    TitleScreen()



class PlayerResourceHand:
    def __init__(self):
        self.brick = 0
        self.grain = 0
        self.lumber = 0
        self.ore = 0
        self.wool = 0
        self.totalResources = 0


class PlayerDevelopmentHand:
    def __init__(self):
        self.knights = 0
        self.roadBuildings = 0
        self.yearOfPlenty = 0
        self.monopolies = 0
        self.victoryPoints = 0
        self.totalDevelopments = 0

    def add_card(self, card):
        if card == 'knight':
            self.knights += 1
        elif card == 'roadBuilding':
            self.roadBuildings += 1
        elif card == 'yearOfPlenty':
            self.yearOfPlenty += 1
        elif card == 'monopoly':
            self.monopolies += 1
        elif card == 'victoryPoint':
            self.victoryPoints += 1
        self.totalDevelopments += 1

    def remove_card(self, card):
        if card == 'knight':
            self.knights -= 1
        elif card == 'roadBuilding':
            self.roadBuildings -= 1
        elif card == 'yearOfPlenty':
            self.yearOfPlenty -= 1
        elif card == 'monopoly':
            self.monopolies -= 1
        elif card == 'victoryPoint':
            self.victoryPoints -= 1
        self.totalDevelopments -= 1


class TradeRatios:
    def __init__(self):
        self.brick = 4
        self.grain = 4
        self.lumber = 4
        self.ore = 4
        self.wool = 4

    def update(self, resource):
        if resource == "Brick":
            self.brick = 2
        elif resource == "Wheat":
            self.grain = 2
        elif resource == "Wood":
            self.lumber = 2
        elif resource == "Ore":
            self.ore = 2
        elif resource == "Sheep":
            self.wool = 2
        elif resource == "None":
            self.brick = min(self.brick, 3)
            self.grain = min(self.grain, 3)
            self.lumber = min(self.lumber, 3)
            self.ore = min(self.ore, 3)
            self.wool = min(self.wool, 3)


class EnemyPlayer:
    def __init__(self, name, color):
        self.name = name
        self.color = color
        self.handSize = '0'
        self.developmentSize = '0'
        self.visibleVictoryPoints = '0'
        self.numRoads = '15'
        self.numSettlements = '5'
        self.numCities = '4'
        self.longestRoad = '0'
        self.largestArmy = '0'

    def update_enemy(self, data):
        self.name = data[0]
        self.color = data[1]
        self.handSize = data[2]
        self.developmentSize = data[3]
        self.visibleVictoryPoints = data[4]
        self.numRoads = data[5]
        self.numSettlements = data[6]
        self.numCities = data[7]
        self.longestRoad = data[8]
        self.largestArmy = data[9]


class Player:
    def __init__(self, name, color, screen_num):
        self.color = color
        self.name = name
        self.numRoads = 15
        self.numSettlements = 5
        self.numCities = 4
        self.longestRoad = 0
        self.claimLongestRoad = False
        self.largestArmy = 0
        self.claimLargestArmy = False
        self.victoryPoints = 0
        self.hiddenVictoryPoints = 0
        self.resourceHand = PlayerResourceHand()
        self.developmentHand = PlayerDevelopmentHand()
        self.bankTrading = TradeRatios()
        self.development_card_played = False
        self.ownedRoads = list()
        self.ownedNodes = list()
        self.screen_num = screen_num

    def get_num_resources(self):
        return self.resourceHand.totalResources

    def get_num_development(self):
        return self.developmentHand.totalDevelopments

    def get_sent_to_enemies(self):
        to_send = ','.join([self.name, self.color,
                            str(self.resourceHand.totalResources), str(self.developmentHand.totalDevelopments),
                            str(self.victoryPoints), str(self.numRoads), str(self.numSettlements), str(self.numCities),
                            str(self.longestRoad), str(self.largestArmy)])
        return to_send

    # todo has problem with joining established roads

    def acquire_road(self, road):
        self.ownedRoads.append(road)
        for edge in self.ownedRoads:
            road_stack = list()
            road_stack.append(edge)
            road_checked = list()
            road_checked.append(edge)

            to_check = self.longest_road_value(road_stack, road_checked)
            if self.longestRoad < to_check:
                self.longestRoad = to_check
        self.numRoads -= 1

    def get_longest_road(self, enemy_longest_road):
        if enemy_longest_road < 5 and self.longestRoad >= 5 and not self.claimLongestRoad:
            self.claimLongestRoad = True
            self.victoryPoints += 2
        elif 5 <= enemy_longest_road < self.longestRoad and not self.claimLongestRoad:
            self.claimLongestRoad = True
            self.victoryPoints += 2
            return True
        return False

    def lose_longest_road(self, enemy_longest_road):
        if enemy_longest_road > self.longestRoad and self.claimLongestRoad:
            self.claimLongestRoad = False
            self.victoryPoints -= 2

    def get_largest_army(self, enemy_largest_army):
        if enemy_largest_army < 3 and self.largestArmy >= 3 and not self.claimLargestArmy:
            self.claimLargestArmy = True
            self.victoryPoints += 2
        elif 3 <= enemy_largest_army < self.largestArmy and not self.claimLargestArmy:
            self.claimLargestArmy = True
            self.victoryPoints += 2
            return True
        return False

    def lost_largest_army(self, enemy_largest_army):
        if enemy_largest_army > self.largestArmy and self.claimLargestArmy:
            self.claimLargestArmy = False
            self.victoryPoints -= 2

    def longest_road_value(self, road_stack, road_checked, length=1):
        current = road_stack.pop()
        to_return = length
        for corner in current.corners:
            for edge in corner.edges:
                if edge is not None and (edge not in road_checked) and (edge.color == self.color):
                    road_stack.append(edge)
                    road_checked.append(edge)
                    length_check = self.longest_road_value(road_stack, road_checked, length + 1)
                    if to_return < length_check:
                        to_return = length_check
        return to_return

    def acquire_node(self, node):
        self.ownedNodes.append(node)
        self.numSettlements -= 1
        self.victoryPoints += 1

    def acquire_city(self):
        self.numCities -= 1
        self.numSettlements += 1
        self.victoryPoints += 1

    def add_resource(self, update_frame):
        self.resourceHand.brick += update_frame[0]
        self.resourceHand.grain += update_frame[1]
        self.resourceHand.lumber += update_frame[2]
        self.resourceHand.ore += update_frame[3]
        self.resourceHand.wool += update_frame[4]

        self.resourceHand.totalResources += sum(update_frame)



# Configurable options:
resolution_selection = 'C'

# Constants

RES_SCALE_DICT = {
    'A': ((1920, 1080), 1),
    'B': ((1200, 800), 75 / 100),
    'C': ((1600, 900), 83 / 100),
    'D': ((960, 540), 50 / 100),
    'E': ((1280, 720), 69 / 100)
}

FRAME_RATE = 60
RESOLUTION, PROTOTYPE_SCALE = RES_SCALE_DICT[resolution_selection]
SPACING = 3
FULLSCREEN = False
SCALE = 5 / 16
HEX_SIZE = 600 * SCALE * PROTOTYPE_SCALE
ACTION_SIZE = (int(70 * PROTOTYPE_SCALE), int(98 * PROTOTYPE_SCALE))
CARD_SIZE = (int(70 * PROTOTYPE_SCALE), int(98 * PROTOTYPE_SCALE))
WINDOW_CENTER = (RESOLUTION[0] / 2, RESOLUTION[1] / 2)

INFO_WIDTH = 6 * ACTION_SIZE[0] + 7 * SPACING
BANK_HEIGHT = 100  # todo scale this
BANK_CARD_WIDTH = (INFO_WIDTH - 8 * SPACING - BANK_HEIGHT) / 6
BANK_HALF_HEIGHT = (BANK_HEIGHT - 3 * SPACING) / 2
PLAYER_HEIGHT = int(4 * 6 / 5 * ACTION_SIZE[0])
PLAYER_SECTION_WIDTH = int((INFO_WIDTH - 4 * SPACING) / 5)
PLAYER_QUARTER_HEIGHT = int((PLAYER_HEIGHT - 2 * SPACING) / 4)

# SCALE = 5 / 16
# HEX_SIZE = 600 * SCALE
# WINDOW_CENTER = (RESOLUTION[0] / 2, RESOLUTION[1] / 2)



import pygame
import pygame_gui as pg_g
from frontiersman.client.GuiConstants import CARD_SIZE, SPACING

CARDS = ['Brick', 'Ore', 'Sheep', 'Wheat', 'Wood']


class CardPickerGui:
    def __init__(self, top_left, manager, callback, container=None):
        self.manager = manager
        self.callback = callback
        self.panel = pg_g.elements.UIPanel(
            relative_rect=pygame.Rect(top_left,
                                      ((CARD_SIZE[0] + SPACING) * 5 + 10 * SPACING, CARD_SIZE[1] + 10 * SPACING)),
            starting_layer_height=1,
            manager=self.manager,
            container=container,
        )
        self.cards = []

        for i, card in enumerate(CARDS):
            self.cards.append(pg_g.elements.UIButton(
                relative_rect=pygame.Rect((i * (CARD_SIZE[0] + SPACING) + 4 * SPACING, 4 * SPACING), CARD_SIZE),
                text="",
                manager=self.manager,
                container=self.panel,
                object_id="#" + card
            ))

    def kill(self):
        self.panel.kill()

    def handle_ui_button_pressed(self, event):
        try:
            index = self.cards.index(event.ui_element)
            self.callback(CARDS[index])
        except ValueError:
            pass



import pygame
import pygame_gui as pg_g

from frontiersman.client.GuiConstants import SPACING, INFO_WIDTH

WIDTH = INFO_WIDTH
HEIGHT = INFO_WIDTH / 5
TITLE_SPACING = 28


class PlayerSelector(pg_g.elements.UIWindow):
    def __init__(self, top_left, enemy_players, manager, text, callback_success, callback_close=None):
        size = (WIDTH + 3 * SPACING, (HEIGHT + SPACING) * (len(enemy_players) + 1) + SPACING + 2 * TITLE_SPACING)
        super().__init__(pygame.Rect(top_left, size), manager)
        self.enemy_players = enemy_players
        self.manager = manager
        self.text = text
        self.callback_success = callback_success
        self.callback_close = callback_close

        if self.callback_close is None:
            self.close_window_button.disable()

        self.info = pg_g.elements.UITextBox(self.text, pygame.Rect(0, 0, WIDTH, HEIGHT), self.manager, container=self)

        self.buttons = []
        for i, enemy_player in enumerate(enemy_players):
            self.buttons.append(pg_g.elements.UIButton(
                relative_rect=pygame.Rect(0, (i + 1) * (HEIGHT + SPACING), WIDTH, HEIGHT),
                manager=self.manager,
                text=enemy_player.name,
                object_id='#' + enemy_player.color.capitalize(),
                container=self
            ))

    def handle_ui_button_pressed(self, event):
        if event.ui_element in self.buttons:
            index = self.buttons.index(event.ui_element)
            self.callback_success(self.enemy_players[index])

    def process_event(self, event):
        if self.callback_close is not None and event.type == pygame.USEREVENT and event.user_type == 'ui_button_pressed' and event.ui_element == self.close_window_button:
            self.callback_close()
        super().process_event(event)



class Bank:
    def __init__(self):
        self.brick = 19
        self.grain = 19
        self.lumber = 19
        self.ore = 19
        self.wool = 19
        self.totalResources = 95
        self.totalDevelopment = 20
        '''
        self.knight = 14
        self.roadBuilding = 2
        self.yearOfPlenty = 2
        self.monopoly = 2
        self.victoryPoint = 5
        '''

    def get_dev_card(self):
        self.totalDevelopment -= 1

    def update(self, update_frame):
        print(update_frame)
        print(self.brick , self.grain , self.lumber ,self.ore , self.wool)
        self.brick += update_frame[0]
        self.grain += update_frame[1]
        self.lumber += update_frame[2]
        self.ore += update_frame[3]
        self.wool += update_frame[4]
        print(self.brick , self.grain , self.lumber ,self.ore , self.wool)
        self.totalResources = self.brick + self.grain + self.lumber + self.ore + self.wool
    


import asyncio
import os
import queue
import random
import threading
from queue import Queue
from typing import Any

from aiohttp import web

loop = asyncio.get_event_loop()
running_games = {}
chars = ['2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'J', 'K',
         'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e',
         'f', 'g', 'h', 'j', 'k', 'm', 'n', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z']


def make_code(length, existing_list):
    code = ''
    for i in range(0, length):
        code = code + random.choice(chars)
    if code in existing_list:
        code = make_code(length, existing_list)
    else:
        return code


def game_over(code):
    try:
        del running_games[code]
        print(running_games.keys())
    except KeyError:
        print('Key', code, 'not found')


class client:
    def __init__(self, name, id2, host):
        self.name = name
        self.id = id2
        self.color = ''
        self.is_host = host
        self.queue = asyncio.Queue()

    def set_data(self, color, screen_num):
        self.color = color
        self.screen_num = screen_num


# from frontiersman.client import ClientHelper
class ServerThread:

    def __init__(self, code):
        self.code = code
        self.player_count = 0
        self.game_code = code
        self.players = {}
        self.started = False
        self.winner = False
        self.queue = queue.Queue()
        self.color_list = ['red', 'cyan', 'orange', 'blue', 'green', 'pink', 'yellow']
        random.shuffle(self.color_list)
        self.resource_list = ['Wheat'] * 4 + \
                             ['Sheep'] * 4 + \
                             ['Ore'] * 3 + \
                             ['Brick'] * 3 + \
                             ['Wood'] * 4 + \
                             ['Desert'] * 1
        self.number_list = [2, 12]
        for index in range(3, 12):
            if index == 7:
                pass
            else:
                self.number_list.append(index)
                self.number_list.append(index)
        self.port_list = ['Wheat'] + \
                         ['Sheep'] + \
                         ['Ore'] + \
                         ['Brick'] + \
                         ['Wood'] + \
                         ['None'] * 4
        self.developmentDeck = ['knight'] * 15 + \
                               ['roadBuilding'] * 2 + \
                               ['yearOfPlenty'] * 2 + \
                               ['monopoly'] * 2 + \
                               ['victoryPoint'] * 5
        random.shuffle(self.developmentDeck)
        random.shuffle(self.number_list)
        random.shuffle(self.resource_list)
        random.shuffle(self.port_list)
        # asyncio.run(self.run())

    def add_player(self, player_object):
        player_object.set_data(self.color_list.pop(0), self.player_count)
        self.players[player_object.id] = player_object
        self.player_count += 1

    def run(self):
        asyncio.run(self.async_run())

    def handle_incoming(self, message):
        self.queue.put(message)

    def game_over(self):
        self.winner = True

    async def async_run(self):

        def dice_roll():
            return str(random.randint(1, 6)) + ',' + str(random.randint(1, 6))

        async def sendtoall(message):
            for key in self.players:
                asyncio.run_coroutine_threadsafe(self.players[key].queue.put(message), loop)

        async def sendtoallButOne(message, cli_code):
            for key in self.players:
                if key != cli_code:
                    asyncio.run_coroutine_threadsafe(self.players[key].queue.put(message), loop)

        async def sendtoOne(message, cli_code):
            asyncio.run_coroutine_threadsafe(self.players[cli_code].queue.put(message), loop)

        def readfromBuffer():
            while self.queue.empty():
                continue
            msg = self.queue.get()
            self.queue.task_done()
            print("test", msg)
            return msg

        def get_key_from_color(color):
            for key in self.players.keys():
                if self.players[key].color == color:
                    return key

        async def quit():
            for key in keyorder:
                await sendtoall("quit")  # turn this into who won for the victory screen, obv
            game_over(self.code)

        # print('Waitiing for a Connection..')
        '''
        res = readfromBuffer(Client)
        name = res
        print(name)
        Client.send(str.encode("host\n"))
        clients.append(client(name, address, Client, Colors[0]))
        Colors.pop(0)
        #numplayers = readfromBuffer(Client)
        #print(int(numplayers))
        for x in range(int(numplayers) - 1):
            Client, address = ServerSocket.accept()
            res = readfromBuffer(Client)
            name = res
            print(name)
            # for cli in clients:
            #    i =0
            #    cli.socket.send(str.encode(name+" connected\n"))
            clients.append(client(name, address, Client, Colors[0]))
            Colors.pop(0)
        
        for i in range(0, len(clients)):
            sendstring = "color,"
            sendstring = sendstring + clients[i].color+','+str(i) + '\n'
            clients[i].socket.send(str.encode(sendstring))
        '''
        # board randomizer

        numberstring = 'board|' + ','.join([str(elem) for elem in self.number_list]) + '|' + ','.join(
            self.resource_list) + '|' + ','.join(self.port_list)
        await sendtoall(numberstring)
        keyorder = list(self.players.keys())
        # for key in keyorder:
        # await sendtoallButOne("enemy," + self.players[key].name + "," + self.players[key].color, key)
        for key in keyorder:
            # cli.socket.send(str.encode("startgui\n"))
            await sendtoOne("startgui", key)
            message = readfromBuffer()
            if message == "quit":
                await quit()
                return

        # setup

        random.shuffle(keyorder)
        for key in keyorder:
            await sendtoOne("set", key)
            print("test")
            coordinates = readfromBuffer()
            print(coordinates)
            if coordinates == "quit":
                await quit()
                return
            await sendtoallButOne("set," + coordinates + "," + self.players[key].color, key)
            message = readfromBuffer()
            if message == "quit":
                await quit()
                return
            await sendtoallButOne(message, key)
            await sendtoOne("startroad", key)
            coordinates = readfromBuffer()
            if coordinates == "quit":
                await quit()
                return
            await sendtoallButOne("road," + coordinates + "," + self.players[key].color, key)
            message = readfromBuffer()
            if message == "quit":
                await quit()
                return
            await sendtoallButOne(message, key)

        for key in reversed(keyorder):
            await sendtoOne("set", key)
            coordinates = readfromBuffer()
            if coordinates == "quit":
                await quit()
                return
            await sendtoallButOne("set," + coordinates + "," + self.players[key].color, key)
            message = readfromBuffer()
            if message == "quit":
                await quit()
                return
            await sendtoallButOne(message, key)
            await sendtoOne("startroad", key)
            coordinates = readfromBuffer()
            if coordinates == "quit":
                await quit()
                return
            await sendtoallButOne("road," + coordinates + "," + self.players[key].color, key)
            message = readfromBuffer()
            if message == "quit":
                await quit()
                return
            await sendtoallButOne(message, key)

        for key in keyorder:
            await sendtoOne('getstart', key)
            message = readfromBuffer()
            if message == "quit":
                await quit()
                return
            await sendtoallButOne(message, key)
            message = readfromBuffer()
            if message == "quit":
                await quit()
                return
            await sendtoallButOne(message, key)
        while not self.winner:
            for key in keyorder:
                dice = dice_roll()
                if int(dice.split(',')[0]) + int(dice.split(',')[1]) == 7:
                    await sendtoall('dice,' + dice)
                    await sendtoall('discard')
                    for x in range(0, len(keyorder)):
                        message = readfromBuffer()
                        if message == "quit":
                            await quit()
                            return
                        if message == "noneed":
                            continue
                        else:
                            res = message.split('|')
                            key4 = get_key_from_color(res[0])
                            await sendtoallButOne(res[1], key4)
                            await sendtoallButOne(res[2], key4)
                else:
                    # await sendtoall('dice,' + dice)
                    for key2 in keyorder:
                        await sendtoOne('dice,' + dice, key2)
                        print('dice,' + dice, key2)
                        message = readfromBuffer()
                        if message == "quit":
                            await quit()
                            return
                        await sendtoallButOne(message, key2)
                        message = readfromBuffer()
                        if message == "quit":
                            await quit()
                            return
                        await sendtoallButOne(message, key2)
                if int(dice.split(',')[0]) + int(dice.split(',')[1]) == 7:
                    await sendtoOne("robber", key)
                else:
                    await sendtoOne("turn", key)
                while True:
                    message = readfromBuffer()
                    if message == "quit":
                        await quit()
                        return
                    # print(message)
                    if message == "end":
                        message = readfromBuffer()
                        if message == "quit":
                            await quit()
                            return
                        await sendtoallButOne(message, key)
                        break
                    elif message.split(',')[0] == "winner":
                        self.winner = True
                        # print(message)
                        break
                    elif message.split('|')[0] == "tradeoffer":
                        await sendtoallButOne(message, key)
                        trade_list = 'tradewith'
                        for x in range(0, len(keyorder) - 1):
                            message = readfromBuffer()
                            if message == "quit":
                                await quit()
                                return
                            if message.split(',')[0] == "deny":
                                continue
                            elif message.split(',')[0] == "accept":
                                trade_list += ',' + message.split(',')[1]
                        await sendtoOne(trade_list, key)
                        if trade_list != 'tradewith':
                            message = readfromBuffer()
                            if message == "quit":
                                await quit()
                                return
                            if message != 'tradecancel':
                                await sendtoOne("trade", get_key_from_color(message.split(',')[1]))
                                message = readfromBuffer()
                                if message == "quit":
                                    await quit()
                                    return
                                await sendtoallButOne(message, get_key_from_color(message.split(',')[2]))
                                message = readfromBuffer()
                                if message == "quit":
                                    await quit()
                                    return
                                await sendtoallButOne(message, get_key_from_color(message.split(',')[2]))
                            else:
                                await sendtoallButOne(message, key)
                    elif message.split(',')[0] == 'steal':
                        for key2 in self.players.keys():
                            if self.players[key2].color == message.split(',')[1]:
                                await sendtoOne('steal', key2)
                                stolen_message = readfromBuffer()
                                if stolen_message == "quit":
                                    await quit()
                                    return
                                enemy_update = readfromBuffer()
                                if enemy_update == "quit":
                                    await quit()
                                    return
                                await sendtoOne(stolen_message, key)
                                for key3 in self.players.keys():
                                    if key3 != key2 and key3 != key:
                                        await sendtoOne(enemy_update, key3)
                    elif message == "dev":
                        card = self.developmentDeck.pop(0)
                        await sendtoOne(card, key)
                    else:
                        await sendtoallButOne(message, key)
                await sendtoOne("notturn", key)
                # message=readfromBuffer(cli.socket)
                # print(message)
                # await sendtoallButOne(message, cli)

        # game loop
        for key in keyorder:
            await sendtoall("quit")  # turn this into who won for the victory screen, obv
        game_over(self.code)

    '''
    except(ConnectionResetError):
        {
            print("A player has disconnected")
        }
    '''


async def join(request):
    data = await request.post()
    code = request.match_info.get('game_id', "fail")
    if code == "fail" or code not in running_games.keys():
        return web.Response(text="game not found")
    else:
        # players=running_games
        if not running_games[code].started:
            player_code = make_code(15, running_games[code].players.keys())
            running_games[code].add_player(client(data['name'], player_code, False))
            enemystring = ''
            for key in running_games[code].players.keys():
                if key != player_code:
                    asyncio.run_coroutine_threadsafe(running_games[code].players[key].queue.put(
                        "enemy," + data['name'] + "," + running_games[code].players[player_code].color), loop)
                    enemystring += '|' + running_games[code].players[key].name + ',' + running_games[code].players[
                        key].color
            return web.Response(text=player_code + ',' + running_games[code].players[player_code].color + ',' + str(
                running_games[code].players[player_code].screen_num) + enemystring)
        else:
            return web.Response(text='game already started')


async def make_game(request):
    data = await request.post()
    game_code = make_code(5, running_games.keys())
    player_code = make_code(15, [])
    running_games[game_code] = ServerThread(game_code)
    running_games[game_code].add_player(client(data['name'], player_code, True))
    return web.Response(
        text=game_code + ',' + player_code + ',' + running_games[game_code].players[player_code].color + ',' + str(
            running_games[game_code].players[player_code].screen_num))


async def get_data(request):
    code = request.match_info.get('game_id', "fail")
    if code == "fail" or code not in running_games.keys():
        return web.Response(text="game not found")
    player_code = request.match_info.get('player_id', "fail")
    if player_code == "fail" or player_code not in running_games[code].players.keys():
        return web.Response(text="player not found")
    msg = await running_games[code].players[player_code].queue.get()
    return web.Response(text=msg)
async def kill_all_games(request):
    data = await request.post()
    if data['code']=='double_snap':
        for key in running_games.keys():
            game_over(key)
async def game_update(request):
    data = await request.post()
    code = request.match_info.get('game_id', "fail")
    if code == "fail" or code not in running_games.keys():
        return web.Response(text="game not found")
    player_code = request.match_info.get('player_id', "fail")
    if player_code == "fail" or player_code not in running_games[code].players.keys():
        return web.Response(text="player not found")
    #
    print('data is', data['msg'])
    if data['msg'] == "start":
        if running_games[code].players[player_code].is_host and not running_games[code].started:
            print('send:', data['msg'])
            threading.Thread(target=running_games[code].run, ).start()
            running_games[code].started = True
        else:
            web.Response(text="you're a cheater")
    elif data['msg'] == "prestart":
        if not running_games[code].started:
            for key in running_games[code].players.keys():
                asyncio.run_coroutine_threadsafe(running_games[code].players[key].queue.put("closelisten"), loop)
    elif data['msg'] == "leave":
        color = running_games[code].players[player_code].color
        if running_games[code].players[player_code].is_host:
            for key in running_games[code].players.keys():
                asyncio.run_coroutine_threadsafe(running_games[code].players[key].queue.put("leave"), loop)
            game_over(code)
        else:
            for key in running_games[code].players.keys():
                if key != player_code:
                    asyncio.run_coroutine_threadsafe(running_games[code].players[key].queue.put("enemyleave," + color),
                                                     loop)
            asyncio.run_coroutine_threadsafe(running_games[code].players[player_code].queue.put("quit"), loop)
            del running_games[code].players[player_code]
            running_games[code].color_list.append(color)
            print(running_games[code].players.keys())
    elif data['msg'] == "quit":
        for key in running_games[code].players.keys():
            asyncio.run_coroutine_threadsafe(running_games[code].players[key].queue.put("quit"), loop)
        running_games[code].handle_incoming(data['msg'])

    else:
        running_games[code].handle_incoming(data['msg'])
    return web.Response(text="sucessful post")


# def hello(request):
def main():
    app = web.Application()
    app.router.add_route('POST', '/create', make_game)
    app.router.add_route('POST', '/join/{game_id}', join)
    app.router.add_route('GET', '/{game_id}/{player_id}/get', get_data)
    app.router.add_route('POST', '/{game_id}/{player_id}/post', game_update)
    app.router.add_route('POST','/killgames', kill_all_games)
    f = loop.create_server(app.make_handler(), '0.0.0.0', os.environ.get('PORT', '5000'))
    srv = loop.run_until_complete(f)
    print('serving on', srv.sockets[0].getsockname())
    try:
        loop.run_forever()
    except KeyboardInterrupt:
        pass


if __name__ == "__main__":
    main()



import subprocess
import sys

import pygame
import pygame_gui
from frontiersman.gameboard.Board import *
from pygame_gui.elements import UITextEntryLine

sys.path.insert(0, '../src')


# os.environ['SDL_AUDIODRIVER'] = 'dsp'
# os.putenv("DISPLAY", "localhost:0")
# os.environ['SDL_AUDIODRIVER'] = 'x11'


class startup:

    def __init__(self):
        self.children = []
        self.startup()

    def open_client(self):
        exec(open('client.py').read())

    def open_server(self):
        exec(open('server.py').read())

    def startup(self):
        WINDOW_WIDTH = 300
        WINDOW_HEIGHT = 200

        pygame.init()

        pygame.display.set_caption('Startup Menu')
        center = (WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2)
        window = pygame.display.set_mode((WINDOW_WIDTH, WINDOW_HEIGHT))
        manager = pygame_gui.UIManager((WINDOW_WIDTH, WINDOW_HEIGHT), './themes/button_theme.json')
        clock = pygame.time.Clock()

        background_color = (61, 120, 180)
        background = pygame.Surface((WINDOW_WIDTH, WINDOW_HEIGHT))
        background.fill(background_color)

        # button dimensions
        button_width = 240
        button_height = 40
        padding = 10

        # text box dimensions
        entry_width = 240
        entry_height = 40
        padding = 10

        position = (int(center[0] - button_width / 2),
                    int(center[1] - button_height - padding - entry_height + 25))
        server_button = pygame_gui.elements.UIButton(relative_rect=pygame.Rect(position, (button_width, button_height)),
                                                     text='Host', manager=manager, object_id='#standard_button')

        position = (int(center[0] - button_width / 2),
                    int(center[1] + padding / 2 + 25))
        client_button = pygame_gui.elements.UIButton(relative_rect=pygame.Rect(position, (button_width, button_height)),
                                                     text='Connect', manager=manager, object_id='#standard_button')

        position = (int(center[0] - button_width / 2),
                    int(center[1] - padding / 2 - entry_height / 2 + 25))

        ip_entry = UITextEntryLine(relative_rect=pygame.Rect(position, (button_width - 80, button_height)),
                                   manager=manager, object_id='#text_entry')

        position = (int(center[0] - button_width / 2 + button_width - 80 + padding),
                    int(center[1] - padding / 2 - entry_height / 2 + 25))

        port_entry = UITextEntryLine(relative_rect=pygame.Rect(position, (80 - padding, button_height)),
                                     manager=manager, object_id='#text_entry')

        # update screen
        pygame.display.flip()

        running = True
        client_flag = False
        server_flag = False

        while running:
            time_delta = clock.tick(60) / 1000.0
            for event in pygame.event.get():
                # close if X is clicked
                if event.type == pygame.QUIT:
                    running = False

                # close if ESC is pressed
                if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                    running = False

                if event.type == pygame.USEREVENT:
                    if event.user_type == pygame_gui.UI_BUTTON_PRESSED:
                        # action if client is pressed
                        if event.ui_element == client_button:
                            client_flag = True
                            # print('Client created')
                        # action if server is pressed
                        if event.ui_element == server_button:
                            server_flag = True

                manager.process_events(event)

            manager.update(time_delta)

            window.blit(background, (0, 0))
            manager.draw_ui(window)

            pygame.display.update()

            if client_flag:
                client_flag = False
                self.children.append(subprocess.Popen([sys.executable, './ClientMain.py']))

            if server_flag:
                server_flag = False
                self.children.append(subprocess.Popen([sys.executable, './server.py']))


test = startup()



import math
import os

import pygame
import pygame.gfxdraw


class RoadHardwareArt:
    @staticmethod
    def generate_road_art(radius, color, angle=0):
        # accepts angle in degrees

        tan_color = (230, 196, 129)

        # create surface to build shape on
        canvas = pygame.Surface((int(radius), int(radius)), pygame.SRCALPHA)
        center = (radius / 2, radius / 2)

        length = radius
        width = radius / 10
        # tip_length = math.tan(math.pi / 6) * width / 2
        tip_length = width * 0.288675135
        # tip_length = 10
        tip_width = width // 2

        '''
        #apply rect body
        body = pygame.Rect(int(center[0] - width / 2),
                           int(center[1] - length / 2 + tip_length),
                           int(width / 2 * 2),
                           int(length - 2 * tip_length + 1))
        pygame.gfxdraw.box(canvas, body, color)

        #apply top trigon
        point = (radius / 2, 0)
        left = (radius / 2 - tip_width, tip_length)
        right = (radius / 2 + tip_width, tip_length)
        pygame.gfxdraw.filled_polygon(canvas, [point, left, right], color)

        #apply bottom trigon
        point = (radius / 2, radius)
        left = (radius / 2 - tip_width, radius - tip_length)
        right = (radius / 2 + tip_width, radius - tip_length)
        pygame.gfxdraw.filled_polygon(canvas, [point, left, right], color)

        '''
        # apply full hex

        # apply top trigon
        top_point = (radius / 2, 0)
        top_left = (radius / 2 - tip_width, tip_length)
        top_right = (radius / 2 + tip_width, tip_length)

        # apply bottom trigon
        bottom_point = (radius / 2, radius)
        bottom_left = (radius / 2 - tip_width, radius - tip_length)
        bottom_right = (radius / 2 + tip_width, radius - tip_length)

        # color shape
        pygame.gfxdraw.filled_polygon(canvas,
                                      [top_left, top_point, top_right, bottom_right, bottom_point, bottom_left],
                                      color)
        # black outline
        pygame.gfxdraw.aapolygon(canvas,
                                 [top_left, top_point, top_right, bottom_right, bottom_point, bottom_left],
                                 tan_color)

        # apply rotate
        canvas = pygame.transform.rotate(canvas, angle)

        return canvas

    @staticmethod
    def generate_road_polygons(road_length, road_ratio):
        # create 3 types of roads, |, /, \
        road_points = []
        road_width = road_length * road_ratio

        road_points.append([
            [-road_width, road_length],
            [road_width, road_length],
            [road_width, -road_length],
            [-road_width, -road_length]
        ])

        angle = math.pi * 5 / 6
        half_angle = math.pi / 2
        end_points = [[road_length * math.cos(angle), road_length * math.sin(angle)],
                      [-1 * road_length * math.cos(angle), -1 * road_length * math.sin(angle)]]
        road_points.append([])

        road_points[1].append([end_points[0][0] - road_width * math.cos(angle + half_angle),
                               end_points[0][1] - road_width * math.sin(angle + half_angle)])
        road_points[1].append([end_points[0][0] - road_width * math.cos(angle - half_angle),
                               end_points[0][1] - road_width * math.sin(angle - half_angle)])
        road_points[1].append([end_points[1][0] - road_width * math.cos(angle - half_angle),
                               end_points[1][1] - road_width * math.sin(angle - half_angle)])
        road_points[1].append([end_points[1][0] - road_width * math.cos(angle + half_angle),
                               end_points[1][1] - road_width * math.sin(angle + half_angle)])

        angle = math.pi / 6
        end_points = [[road_length * math.cos(angle), road_length * math.sin(angle)],
                      [-1 * road_length * math.cos(angle), -1 * road_length * math.sin(angle)]]
        road_points.append([])

        road_points[2].append([end_points[0][0] - road_width * math.cos(angle + half_angle),
                               end_points[0][1] - road_width * math.sin(angle + half_angle)])
        road_points[2].append([end_points[0][0] - road_width * math.cos(angle - half_angle),
                               end_points[0][1] - road_width * math.sin(angle - half_angle)])
        road_points[2].append([end_points[1][0] - road_width * math.cos(angle - half_angle),
                               end_points[1][1] - road_width * math.sin(angle - half_angle)])
        road_points[2].append([end_points[1][0] - road_width * math.cos(angle + half_angle),
                               end_points[1][1] - road_width * math.sin(angle + half_angle)])

        return road_points


class TextureScale:
    @staticmethod
    def scale_resource_tiles(hex_size):

        base_path = os.path.dirname(__file__)

        texture_dictionary = {
            'Desert': pygame.image.load(os.path.join(base_path, '../assets/hexes/desert.png')),
            'Ocean': pygame.image.load(os.path.join(base_path, '../assets/hexes/ocean.png')),
            'Wheat': pygame.image.load(os.path.join(base_path, '../assets/hexes/wheat.png')),
            'Brick': pygame.image.load(os.path.join(base_path, '../assets/hexes/quarry.png')),
            'Ore': pygame.image.load(os.path.join(base_path, '../assets/hexes/mountain.png')),
            'Sheep': pygame.image.load(os.path.join(base_path, '../assets/hexes/plains.png')),
            'Wood': pygame.image.load(os.path.join(base_path, '../assets/hexes/forest.png')),
        }

        for key in texture_dictionary:
            picture = texture_dictionary.get(key)
            texture_dictionary[key] = pygame.transform.scale(picture, (int(hex_size), int(hex_size)))

        return texture_dictionary

    @staticmethod
    def scale_port_tiles(hex_size):

        base_path = os.path.dirname(__file__)

        port_textures = {
            'None': pygame.image.load(os.path.join(base_path, '../assets/default_port.png')),
            'Wheat': pygame.image.load(os.path.join(base_path, '../assets/wheat_port.png')),
            'Brick': pygame.image.load(os.path.join(base_path, '../assets/brick_port.png')),
            'Ore': pygame.image.load(os.path.join(base_path, '../assets/ore_port.png')),
            'Sheep': pygame.image.load(os.path.join(base_path, '../assets/sheep_port.png')),
            'Wood': pygame.image.load(os.path.join(base_path, '../assets/wood_port.png')),
        }

        for key in port_textures:
            picture = port_textures.get(key)
            port_textures[key] = pygame.transform.scale(picture, (int(hex_size), int(hex_size)))

        return port_textures

    @staticmethod
    def scale_cards(card_size):

        base_path = os.path.dirname(__file__)

        card_images = {
            'knight': pygame.image.load(os.path.join(base_path, '../assets/cards/dev/Knight.png')),
            'monopoly': pygame.image.load(os.path.join(base_path, '../assets/cards/dev/Monopoly.png')),
            'roadBuilding': pygame.image.load(os.path.join(base_path, '../assets/cards/dev/Road_Building.png')),
            'yearOfPlenty': pygame.image.load(os.path.join(base_path, '../assets/cards/dev/Year_Of_Plenty.png')),
            'victoryPoint': pygame.image.load(os.path.join(base_path, '../assets/cards/dev/Victory_Point.png')),
            'Brick': pygame.image.load(os.path.join(base_path, '../assets/cards/res/bricks.png')),
            'Ore': pygame.image.load(os.path.join(base_path, '../assets/cards/res/ore.png')),
            'Sheep': pygame.image.load(os.path.join(base_path, '../assets/cards/res/sheep.png')),
            'Wheat': pygame.image.load(os.path.join(base_path, '../assets/cards/res/wheat.png')),
            'Wood': pygame.image.load(os.path.join(base_path, '../assets/cards/res/wood.png'))
        }

        for key in card_images:
            picture = card_images.get(key)
            card_images[key] = pygame.transform.scale(picture, card_size)

        return card_images

    @staticmethod
    def scale_bank(card_size):

        base_path = os.path.dirname(__file__)

        card_images = {
            'Brick': pygame.image.load(os.path.join(base_path, '../assets/cards/res/bricks.png')),
            'Ore': pygame.image.load(os.path.join(base_path, '../assets/cards/res/ore.png')),
            'Sheep': pygame.image.load(os.path.join(base_path, '../assets/cards/res/sheep.png')),
            'Wheat': pygame.image.load(os.path.join(base_path, '../assets/cards/res/wheat.png')),
            'Wood': pygame.image.load(os.path.join(base_path, '../assets/cards/res/wood.png')),
            'Dev': pygame.image.load(os.path.join(base_path, '../assets/cards/res/resource_card_bank_square.png')),
        }

        for key in card_images:
            picture = card_images.get(key)
            card_images[key] = pygame.transform.scale(picture, card_size)

        return card_images

    @staticmethod
    def scale_logos(logo_size):
        base_path = os.path.dirname(__file__)
        logo_images = {
            'army': pygame.image.load(os.path.join(base_path, '../assets/army.png')),
            'city': pygame.image.load(os.path.join(base_path, '../assets/city.png')),
            'num_roads': pygame.image.load(os.path.join(base_path, '../assets/num_roads.png')),
            'settlement': pygame.image.load(os.path.join(base_path, '../assets/settlement.png')),
            'road': pygame.image.load(os.path.join(base_path, '../assets/road.png')),
            'cards': pygame.image.load(os.path.join(base_path, '../assets/cards.png')),
            'star': pygame.image.load(os.path.join(base_path, '../assets/star.png')),
            'bank': pygame.image.load(os.path.join(base_path, '../assets/bank.png')),
        }

        for key in logo_images:
            picture = logo_images[key]
            logo_images[key] = pygame.transform.scale(picture, logo_size)

        return logo_images


class ButtonResources:
    @staticmethod
    def save_scaled_cards(scaled_cards):
        base_path = os.path.dirname(__file__)

        for key in scaled_cards.keys():
            card = scaled_cards[key]
            pygame.image.save(card, os.path.join(base_path, '../assets/scaled/' + key + '.png'))

            darkened_amount = 10
            darkened_card = card.copy()
            dark = pygame.Surface((card.get_width(), card.get_height()), flags=pygame.SRCALPHA)
            dark.fill((darkened_amount, darkened_amount, darkened_amount, 0))
            darkened_card.blit(dark, (0, 0), special_flags=pygame.BLEND_RGBA_SUB)
            pygame.image.save(darkened_card, os.path.join(base_path, '../assets/scaled/' + key + '_disabled.png'))

            lightened_amount = 10
            lightened_card = card.copy()
            light = pygame.Surface((card.get_width(), card.get_height()), flags=pygame.SRCALPHA)
            light.fill((lightened_amount, lightened_amount, lightened_amount, 0))
            lightened_card.blit(light, (0, 0), special_flags=pygame.BLEND_RGBA_ADD)
            pygame.image.save(lightened_card, os.path.join(base_path, '../assets/scaled/' + key + '_hovered.png'))



import pygame
import pygame_gui as pg_g
from frontiersman.client.GuiConstants import CARD_SIZE, SPACING


class CardDiscarder:
    def __init__(self, rect, cards, manager, callback, layer_height, container=None):
        rect = pygame.Rect(rect.left, rect.top - rect.height, rect.width, rect.height * 2)
        self.manager = manager
        self.cards = cards
        self.selected_cards = [False for _ in self.cards]
        self.card_elements = []
        self.callback = callback

        self.panel = pg_g.elements.UIPanel(
            relative_rect=rect,
            starting_layer_height=layer_height,
            manager=self.manager,
            container=container,
        )

        area_width = self.panel.relative_rect.width - CARD_SIZE[0] - 8
        if len(self.cards) == 1:
            offset = 0
        else:
            offset = min(area_width / (len(self.cards) - 1), CARD_SIZE[0] + SPACING)
        start = (area_width - offset * (len(self.cards) - 1)) / 2

        for i, card in enumerate(self.cards):
            self.card_elements.append(pg_g.elements.UIButton(
                relative_rect=pygame.Rect((start + i * offset, self.panel.rect.height / 2), CARD_SIZE),
                text="",
                manager=self.manager,
                container=self.panel,
                object_id="#" + card
            ))

        self.submit_button = pg_g.elements.UIButton(
            relative_rect=pygame.Rect((-CARD_SIZE[0], 0), CARD_SIZE),
            text="submit",
            manager=self.manager,
            container=self.panel,
            anchors={'left': 'right',
                     'right': 'right',
                     'top': 'top',
                     'bottom': 'top'}
        )
        self.submit_button.disable()

    def kill(self):
        self.panel.kill()

    def handle_ui_button_pressed(self, event):
        element = event.ui_element
        if element in self.card_elements:
            index = self.card_elements.index(element)
            self.selected_cards[index] = not self.selected_cards[index]
            if self.selected_cards[index]:
                element.rect.top = self.panel.rect.top + SPACING * 2
            else:
                element.rect.top = self.panel.rect.top + self.panel.rect.height / 2 + SPACING * 2
            element.rebuild()
            count = 0
            for value in self.selected_cards:
                if value:
                    count += 1
            if count == len(self.cards) // 2:
                self.submit_button.enable()
            else:
                self.submit_button.disable()
        elif element == self.submit_button:
            clicked_cards = []
            for i, value in enumerate(self.selected_cards):
                if value:
                    clicked_cards.append(self.cards[i])
            self.callback(clicked_cards)



import pygame
import pygame_gui
from pygame_gui.elements import UITextEntryLine

from frontiersman.ClientMain import client

'''
sys.path.insert(0, '../src')
from pygame_gui.elements import UITextEntryLine
from ClientMain import client
from GameBoard.NodeRoads import *
from GameBoard.Board import *
from client.Client import *
from client.Player import *
'''


# os.environ['SDL_AUDIODRIVER'] = 'dsp'
# os.putenv("DISPLAY", "localhost:0")
# os.environ['SDL_AUDIODRIVER'] = 'x11'


def get_valid_player_input(query_string):
    while True:
        try:
            innie = int(input(query_string))
        except ValueError:
            print("invalid number of players")
            continue
        if 0 < innie <= 4:
            return innie
        elif innie > 4:
            print("Maximum number of players is 4")
            continue
        elif innie < 0:
            print("number of players cannot be negative")


def board_thread():
    client.run()


class ClientStartWithUI:

    def __init__(self):
        self.children = []
        self.user_interface()

    def open_client(self):
        exec(open('client.py').read())

    def user_interface(self):
        WINDOW_WIDTH = 300
        WINDOW_HEIGHT = 200

        Colors = {
            'Red': (127, 0, 0),
            'Cyan': (0, 255, 255),
            'Orange': (255, 106, 0),
            'Blue': (0, 38, 255),
            'Green': (0, 153, 15),
            'Pink': (255, 0, 110),
            'Yellow': (255, 216, 0)
        }

        pygame.init()

        pygame.display.set_caption('Startup Menu')
        center = (WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2)
        window = pygame.display.set_mode((WINDOW_WIDTH, WINDOW_HEIGHT))
        manager = pygame_gui.UIManager((WINDOW_WIDTH, WINDOW_HEIGHT), './themes/button_theme.json')
        clock = pygame.time.Clock()

        background_color = (61, 120, 180)
        background = pygame.Surface((WINDOW_WIDTH, WINDOW_HEIGHT))
        background.fill(background_color)

        # color button dimensions
        button_center = [center[0], center[1] + 50]
        button_size = [30, 30]
        padding = 5
        num_buttons = 7

        colors = list(Colors.keys())

        color_buttons = []

        for index in range(-3, 4):
            position = (int(button_center[0] + index * (padding + button_size[0]) - button_size[0] / 2),
                        int(button_center[1] - padding - 75 - 3))
            pygame_gui.elements.UIButton(relative_rect=pygame.Rect(position, button_size),
                                         text='', manager=manager, object_id='#' + colors[index + 3])

        # button dimensions
        button_width = 240
        button_height = 40
        padding = 10

        # text box dimensions
        entry_width = 240
        entry_height = 40
        padding = 10

        position = (int(center[0] - button_width / 2),
                    int(center[1] + padding / 2 + 25))
        client_button = pygame_gui.elements.UIButton(relative_rect=pygame.Rect(position, (button_width, button_height)),
                                                     text='Connect', manager=manager, object_id='#standard_button')

        position = (int(center[0] - button_width / 2),
                    int(center[1] - padding / 2 - entry_height / 2 + 25))

        ip_entry = UITextEntryLine(relative_rect=pygame.Rect(position, (button_width - 80, button_height)),
                                   manager=manager, object_id='#text_entry')
        ip_entry.set_text('127.0.0.1')

        position = (int(center[0] - button_width / 2 + button_width - 80 + padding),
                    int(center[1] - padding / 2 - entry_height / 2 + 25))

        port_entry = UITextEntryLine(relative_rect=pygame.Rect(position, (80 - padding, button_height)),
                                     manager=manager, object_id='#text_entry')
        port_entry.set_text('1233')

        position = (int(center[0] - button_width / 2 + 3),
                    int(center[1] - padding / 2 - entry_height / 2 + 25 - 65 + 3))

        name_label = pygame_gui.elements.ui_label.UILabel(relative_rect=pygame.Rect(position, (50, button_height - 17)),
                                                          text='Name:', manager=manager, object_id='#text_entry')

        position = (int(center[0] - button_width / 2 + 55 + padding),
                    int(center[1] - padding / 2 - entry_height / 2 + 25 - 65))

        name_entry = UITextEntryLine(relative_rect=pygame.Rect(position, (button_width - 55 - padding, button_height)),
                                     manager=manager, object_id='#text_entry')

        # update screen
        pygame.display.flip()

        running = True
        client_flag = False
        server_flag = False

        while running:
            time_delta = clock.tick(60) / 1000.0
            for event in pygame.event.get():
                # close if X is clicked
                if event.type == pygame.QUIT:
                    running = False

                # close if ESC is pressed
                if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                    running = False

                if event.type == pygame.USEREVENT:
                    if event.user_type == pygame_gui.UI_BUTTON_PRESSED:
                        # action if client is pressed
                        if event.ui_element == client_button:
                            client_flag = True
                            # print('Client created')

                manager.process_events(event)

            manager.update(time_delta)

            window.blit(background, (0, 0))
            manager.draw_ui(window)

            pygame.display.update()

            if client_flag:
                running = False
                client_flag = False
                # self.children.append(subprocess.Popen([sys.executable, './ClientMain.py']))
                pygame.quit()
                start_instance = client(ip_entry.text, port_entry.text, name_entry.text)


if __name__ == "__main__":
    ClientStartWithUI()



import pygame
from typing import Union
from pygame_gui.core import UIElement, IContainerLikeInterface
from pygame_gui.elements import UIWindow, UIImage, UIVerticalScrollBar


# todo resize properly

class HelpImage(UIVerticalScrollBar):
    def __init__(self, ui_manager,
                 container: Union[IContainerLikeInterface, None] = None,
                 parent_element: UIElement = None, ):
        super().__init__(relative_rect=pygame.Rect((769 - 32 - 2, 0 - 2), (32 + 2, 346)),
                         visible_percentage=.62,
                         manager=ui_manager,
                         container=container,
                         parent_element=parent_element)
        self.HELP_IMAGE = pygame.image.load('./assets/help.png')
        self.size = self.HELP_IMAGE.get_size()
        self.background = pygame.Surface(self.size)
        self.background.fill((0, 0, 0))
        self.visible_percentage = .5
        self.manager = ui_manager
        self.background = pygame.Surface(self.size)
        self.background = self.background.convert()

    def draw(self, surface):
        surface.blit(self.background, (0, 0))
        surface.blit(self.HELP_IMAGE, (0, 0 - int(self.scroll_position * 4)))


class HelpWindow(UIWindow):
    def __init__(self, position, ui_manager):
        self.size = (769 + 28, 400)
        image_size = (737, 802) # todo ???
        super().__init__(pygame.Rect(position, self.size), ui_manager,
                         window_display_title='Help Window',
                         object_id='#help_window')

        surface_size = self.get_container().get_size()
        self.surface_element = UIImage(pygame.Rect((0, 0),
                                                   surface_size),
                                       pygame.Surface(surface_size).convert(),
                                       manager=ui_manager,
                                       container=self,
                                       parent_element=self)

        self.help_image = HelpImage(ui_manager, container=self, parent_element=self)

        self.is_active = False

        self.set_blocking(True)

    def update(self, time_delta):
        super().update(time_delta)

        self.help_image.draw(self.surface_element.image)



import pygame_gui as pg_g


class TextDialog(pg_g.elements.UITextBox):
    def __init__(self, rect, manager, max_lines=100):
        super().__init__('', rect, manager)
        self.max_lines = max_lines
        self.lines = []

    def add_line(self, line):
        self.lines.append(line)
        if len(self.lines) == self.max_lines:
            self.lines = self.lines[1:]
        self.html_text = '<br>'.join(reversed(self.lines))
        self.rebuild()




import sys
import pygame
import pygame_gui

sys.path.insert(0, '../frontiersman')

# os.environ['SDL_AUDIODRIVER'] = 'dsp'
# os.putenv("DISPLAY", "localhost:0")
# os.environ['SDL_AUDIODRIVER'] = 'x11'


class MultiplayerMenu:
    def __init__(self):
        pass


class MainMenu:

    def resize(self, window, manager):
        WINDOW_WIDTH = pygame.display.get_window_size()[0]
        WINDOW_HEIGHT = pygame.display.get_window_size()[1]

        Colors = {
            'Red': (127, 0, 0),
            'Cyan': (0, 255, 255),
            'Orange': (255, 106, 0),
            'Blue': (0, 38, 255),
            'Green': (0, 153, 15),
            'Pink': (255, 0, 110),
            'Yellow': (255, 216, 0)
        }

        pygame.init()

        pygame.display.set_caption('Frontiersman Menu')
        center = (WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2)
        clock = pygame.time.Clock()

        background_color = (61, 120, 180)
        background = pygame.Surface((WINDOW_WIDTH, WINDOW_HEIGHT))
        background.fill(background_color)

        # function to add menu buttons to UI manager
        manager.clear_and_reset()

        def menu_button(location, text):
            menu_buttons.append(pygame_gui.elements.UIButton(
                relative_rect=pygame.Rect(location, (button_width, button_height)),
                text=text, manager=manager,
                object_id='#menu_button_square'))

            # the four buttons text

        button_labels = [
            'Singleplayer',
            'Multiplayer',
            'Options',
            'Exit'
        ]
        # button dimensions
        button_width = WINDOW_HEIGHT // 10 * 5
        button_height = WINDOW_HEIGHT // 10
        padding = 0

        menu_buttons = []
        menu_title = []
        pygame.font.init()
        button_font = pygame.font.Font("../data/fonts/impact.ttf", WINDOW_HEIGHT // 30)
        button_surface = pygame.Surface((WINDOW_WIDTH, WINDOW_HEIGHT), pygame.SRCALPHA)

        # add the four buttons
        for index in (0, 1, 2, 3):
            position = (int(center[0] - button_width / 2),
                        int((4 + index) * button_height))
            # menu_button(position, button_labels[index])
            menu_button(position, "")
            # create background button text so its more easily scaled
            button_text = pygame.font.Font.render(button_font, button_labels[index], True, (255, 255, 255), None)
            button_surface.blit(
                button_text,
                (int(center[0] - button_text.get_width() / 2),
                 int((4.5 + index) * button_height - button_text.get_height() / 2))
            )

        # title dimensions
        entry_width = WINDOW_WIDTH // 4 * 3
        entry_height = WINDOW_HEIGHT // 5
        padding = 0

        title_font = pygame.font.Font("../data/fonts/impact.ttf", WINDOW_HEIGHT // 5)

        title_text = pygame.font.Font.render(title_font, "Frontiersman", True, (255, 255, 255), None)

        title_surface = pygame.Surface((WINDOW_WIDTH, WINDOW_HEIGHT), pygame.SRCALPHA)

        title_surface.blit(title_text, (int(WINDOW_WIDTH / 2 - title_text.get_width() / 2),
                                        int(WINDOW_HEIGHT / 5 - title_text.get_height() / 2)))

        # update screen
        pygame.display.flip()
        return window, title_surface, button_surface

    def __init__(self):
        self.children = []
        self.user_interface()

    def open_client(self):
        exec(open('client.py').read())

    def user_interface(self):
        RESOLUTION = "720p"
        RESOLUTION_DICT = {
            "1080p": (1920, 1080),
            "720p": (1280, 720)
        }

        WINDOW_WIDTH = RESOLUTION_DICT.get(RESOLUTION)[0]
        WINDOW_HEIGHT = RESOLUTION_DICT.get(RESOLUTION)[1]

        test_string = "Awesome"

        pygame.init()

        pygame.display.set_caption('Frontiersman Menu')
        center = (WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2)
        window = pygame.display.set_mode((WINDOW_WIDTH, WINDOW_HEIGHT), flags=pygame.RESIZABLE)
        manager = pygame_gui.UIManager((WINDOW_WIDTH, WINDOW_HEIGHT), '../themes/button_theme.json')
        clock = pygame.time.Clock()

        background_color = (61, 120, 180)
        background = pygame.Surface((WINDOW_WIDTH, WINDOW_HEIGHT))
        background.fill(background_color)

        # function to add menu buttons to UI manager
        def menu_button(location, text):
            menu_buttons.append(pygame_gui.elements.UIButton(
                relative_rect=pygame.Rect(location, (button_width, button_height)),
                text=text, manager=manager,
                object_id='#menu_button_square'))

        # the four buttons text
        button_labels = [
            'Singleplayer',
            'Multiplayer',
            'Options',
            'Exit'
        ]
        # button dimensions
        button_width = WINDOW_HEIGHT // 10 * 5
        button_height = WINDOW_HEIGHT // 10
        padding = 0

        menu_buttons = []
        menu_title = []
        pygame.font.init()
        button_font = pygame.font.Font("../data/fonts/impact.ttf", WINDOW_HEIGHT // 30)
        button_surface = pygame.Surface(RESOLUTION_DICT.get(RESOLUTION), pygame.SRCALPHA)

        # add the four buttons
        for index in (0, 1, 2, 3):
            position = (int(center[0] - button_width / 2),
                        int((4 + index) * button_height))
            #menu_button(position, button_labels[index])
            menu_button(position, "")
            #create background button text so its more easily scaled
            button_text = pygame.font.Font.render(button_font, button_labels[index], True, (255, 255, 255), None)
            button_surface.blit(
                button_text,
                (int(center[0] - button_text.get_width() / 2),
                    int((4.5 + index) * button_height - button_text.get_height() / 2))
            )

        # title dimensions
        entry_width = WINDOW_WIDTH // 4 * 3
        entry_height = WINDOW_HEIGHT // 5
        padding = 0

        title_font = pygame.font.Font("../data/fonts/impact.ttf", WINDOW_HEIGHT // 5)

        title_text = pygame.font.Font.render(title_font, "Frontiersman", True, (255, 255, 255), None)

        title_surface = pygame.Surface((WINDOW_WIDTH, WINDOW_HEIGHT), pygame.SRCALPHA)

        title_surface.blit(title_text, (int(WINDOW_WIDTH / 2 - title_text.get_width() / 2),
                                        int(WINDOW_HEIGHT / 5 - title_text.get_height() / 2)))

        # update screen
        pygame.display.flip()

        running = True
        client_flag = False
        server_flag = False

        while running:
            time_delta = clock.tick(60) / 1000.0
            for event in pygame.event.get():
                # close if X is clicked
                if event.type == pygame.QUIT:
                    running = False
                if event.type == pygame.VIDEORESIZE:
                    #initialize_gui()
                    pass
                # close if ESC is pressed
                if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                    running = False

                if event.type == pygame.USEREVENT:
                    if event.user_type == pygame_gui.UI_BUTTON_PRESSED:
                        # action if client is pressed
                        if event.ui_element == menu_buttons[3]:
                            client_flag = True
                            # print('Client created')

                manager.process_events(event)

            manager.update(time_delta)

            window.blit(background, (0, 0))
            window.blit(button_surface, (0, 0))
            window.blit(title_surface, (0, 0))
            manager.draw_ui(window)

            pygame.display.update()

            if client_flag:
                running = False
                client_flag = False
                # self.children.append(subprocess.Popen([sys.executable, './ClientMain.py']))
                pygame.quit()
                #start_instance = client(ip_entry.text, port_entry.text, name_entry.text)


if __name__ == "__main__":
    MainMenu()



import sys, os

import pygame
import pygame.gfxdraw
import pygame_gui
from pygame_gui.core.utility import clipboard_copy

sys.path.insert(0, '../../frontiersman')
from frontiersman.main.PopUp import PopUp
# sys.path.insert(0, '../../frontiersman')
from pygame_gui.elements import UITextEntryLine
from frontiersman.gameboard.Board import *


class GameLobby:
    def create_window(self):
        WINDOW_WIDTH = self.size[0]
        WINDOW_HEIGHT = self.size[1]

        pygame.init()

        pygame.display.set_caption('Startup Menu')
        os.environ['SDL_VIDEODRIVER'] = 'x11'
        self.window = pygame.display.set_mode((WINDOW_WIDTH, WINDOW_HEIGHT))

        background_color = (61, 120, 180)
        background = pygame.Surface((WINDOW_WIDTH, WINDOW_HEIGHT))
        background.fill(background_color)
        self.window.blit(background, (0, 0))
        pygame.display.update()

    def __init__(self, size, host=False, callbacks=None, manager=None):
        # print(os.getcwd())
        self.name = 'Nick'
        self.code = None
        self.font = None
        self.small_font = None
        self.large_font = None
        self.state = 'menu'  # menu, join, create
        self.prompt_color = (255, 255, 255)
        self.text_color = (0, 0, 0)
        self.temp_text_color = (127, 127, 127)
        self.size = size
        self.width = self.size[0]
        self.height = self.size[1]
        self.init_pygame()
        self.is_host = host
        self.players = []
        # self.is_host = True
        self.notify_window_up = False
        self.notify_windows = []
        self.button_list = []
        self.text_entry_list = []
        self.lobby_status = []
        self.lobby_code = ''
        self.player_code = ''
        self.board = None
        self.leave_lobby = False
        self.quit_gui = False
        self.to_menu = False
        self.run_game = False
        self.host_left = False
        self.client_instance = None

        if manager is None:
            self.manager = pygame_gui.UIManager(size, './themes/menu_theme.json')
        else:
            self.manager = manager

        if callbacks is None:
            self.callbacks = []

            def callback_join():
                print('Join callback')

            def callback_quit():
                print('quit callback')

            def callback_create():
                print('create callback')

            def callback_leave():
                print('leave callback')

            self.callbacks.append(callback_join)
            self.callbacks.append(callback_quit)
            self.callbacks.append(callback_create)
            self.callbacks.append(callback_leave)
        else:
            self.callbacks = callbacks

        self.window = pygame.Surface(self.size)

        self.graphics = pygame.Surface(self.size)

        self.create_window()

        self.state = 'menu'

        self.change_state()

        self.animations = []

        self.running = True
        clock = pygame.time.Clock()

        while self.running is True:
            if self.run_game:
                self.state = 'game'
                self.change_state()
                self.run_game = False
            elif self.to_menu:
                print("to menu!")
                self.lobby_code = ''
                self.player_code = ''
                self.name = ''
                self.color = ''
                self.screen_num = ''
                self.players = []
                self.lobby_status = []
                self.state = 'menu'
                self.change_state()
                self.to_menu = False
            elif self.host_left:
                print("to menu!")
                self.lobby_code = ''
                self.player_code = ''
                self.name = ''
                self.color = ''
                self.screen_num = ''
                self.players = []
                self.lobby_status = []
                self.state = 'menu'
                self.change_state()
                if not self.is_host:
                    self.create_notification("Host left, lobby terminated")
                self.host_left = False
            
            if self.state == "game":
                self.client_instance.update_frame()
                if not self.client_instance.running:
                    print('testing bitch')
                    break
            else:
                try:
                    time_delta = clock.tick(60) / 1000.0
                    if self.leave_lobby:
                        self.lobby_code = ''
                        self.player_code = ''
                        self.name = ''
                        self.color = ''
                        self.screen_num = ''
                        self.players = []
                        self.lobby_status = []
                        
                        self.state = 'menu'
                        self.change_state()
                        self.leave_lobby = False

                        # pygame.display.quit()
                        # pygame.quit()
                        # break
                    for event in pygame.event.get():
                        # close if X is clicked
                        # print(event)
                        if event.type == pygame.QUIT:
                            # self.running = False
                            if self.state == 'lobby':
                                self.callbacks[2](self)
                            self.state = 'quit'
                            self.change_state()
                        if event.type == pygame.KEYDOWN:
                            if event.key == pygame.key.key_code('0'):
                                print('0')
                                self.graphics_banner()

                        self.manager.process_events(event)
                        if event.type == pygame.USEREVENT:
                            if self.notify_window_up:
                                if event.user_type == pygame_gui.UI_WINDOW_CLOSE:
                                    self.notify_windows.remove(event.ui_element)
                                    if len(self.notify_windows) == 0:
                                        self.notify_window_up = False

                            if self.state == 'create':
                                if event.user_type == pygame_gui.UI_BUTTON_PRESSED:
                                    # action if client is pressed
                                    if len(self.button_list) != 0:
                                        if event.ui_element == self.button_list[0]:
                                            self.state = 'menu'
                                            self.change_state()
                                        elif event.ui_element == self.button_list[1]:
                                            self.players = []
                                            self.lobby_status = []
                                            self.name = self.text_entry_list[0].get_text()
                                            error_strings = []
                                            error = False
                                            if self.name == '':
                                                error_strings.append('Missing name')
                                                error = True
                                            if error:
                                                self.create_notification(error_strings, True)
                                            else:
                                                if self.callbacks[0](self):
                                                    self.state = 'lobby'
                                                    self.change_state()
                                                else:
                                                    # do nothing I guess
                                                    pass
                            elif self.state == 'lobby':
                                if event.user_type == pygame_gui.UI_BUTTON_PRESSED:
                                    # action if client is pressed
                                    if len(self.button_list) != 0:
                                        if event.ui_element == self.button_list[0]:
                                            self.leave_lobby = True
                                            self.players = []
                                            self.callbacks[2](self)
                                        elif self.is_host and event.ui_element == self.button_list[1]:
                                            # self.state = 'start_game'
                                            # self.change_state()
                                            self.callbacks[3](self)
                            elif self.state == 'join':
                                if event.user_type == pygame_gui.UI_BUTTON_PRESSED:
                                    # action if client is pressed
                                    if len(self.button_list) != 0:
                                        if event.ui_element == self.button_list[0]:
                                            self.players = []
                                            self.lobby_status = []
                                            self.name = self.text_entry_list[0].get_text()
                                            self.lobby_code = self.text_entry_list[1].get_text()
                                            error_strings = []
                                            error = False
                                            if self.name == '':
                                                error_strings.append('Missing name')
                                                error = True
                                            if self.lobby_code == '':
                                                error_strings.append('Missing code')
                                                error = True
                                            if error:
                                                self.create_notification(error_strings, True)
                                            else:
                                                if self.callbacks[1](self):
                                                    self.state = 'lobby'
                                                    self.change_state()
                                                else:
                                                    self.create_notification("Didn't connect")
                                        elif event.ui_element == self.button_list[1]:
                                            self.state = 'menu'
                                            self.change_state()
                            elif self.state == 'menu':
                                if event.user_type == pygame_gui.UI_BUTTON_PRESSED:
                                    # action if client is pressed
                                    if len(self.button_list) != 0:
                                        if event.ui_element == self.button_list[0]:
                                            self.state = 'join'
                                            self.is_host = False
                                            self.change_state()
                                        elif event.ui_element == self.button_list[1]:
                                            self.state = 'create'
                                            self.is_host = True
                                            self.change_state()
                                        elif event.ui_element == self.button_list[2]:
                                            self.state = 'quit'
                                            self.change_state()

                    self.manager.update(time_delta)
                    self.window.blit(self.graphics, (0, 0))

                    if len(self.animations) > 0:
                        for animations in self.animations:
                            if animations[1] > 0:
                                self.window.blit(animations[0], (0, 0))
                                animations[0].set_alpha(animations[2])
                                animations[2] -= animations[3]
                                animations[1] -= 1
                            else:
                                self.animations.remove(animations)
                    self.manager.draw_ui(self.window)

                    pygame.display.update()
                except AttributeError:
                    print('oof')
        pygame.display.quit()
        pygame.quit()
        sys.exit(0)

    def add_player(self, player):
        self.players.append(player)
        self.lobby_status.append(player.name + ' Joined')
        self.change_state()

    def remove_player(self, player):
        self.players.remove(player)
        self.lobby_status.append(player.name + ' Left')
        self.change_state()

    def set_lobby_code(self, code):
        self.lobby_code = code
        self.change_state()

    def change_state(self):
        background_color = (61, 120, 180)
        background = pygame.Surface(self.size)
        background.fill(background_color)
        self.window.blit(background, (0, 0))

        self.manager.clear_and_reset()
        self.clear_button_list()
        self.clear_text_list()

        flag = False

        if self.state == 'join':
            self.graphics_join()
            flag = True

        if self.state == 'create':
            self.graphics_create()
            flag = True

        if self.state == 'lobby':
            self.graphics_lobby()
            flag = True

        if self.state == 'menu':
            # self.clear_text_list()
            self.graphics_main()
            flag = True

        if self.state == 'game':
            flag = True
            pass
        if self.state == 'game':
            self.client_instance.run(self.board)
            flag = True
        if self.state == 'quit':
            self.running = False
            flag = True
        if self.state == 'start':
            self.running = False
            flag = True

        # error handling crash to main menu
        if flag is False:
            print(self.state + ' is not a valid gui state, returning to main menu')
            self.state = 'main'
            self.graphics_main()

        #pygame.display.update()

    def create_10x10(self):
        graphics = pygame.Surface(self.size, flags=pygame.SRCALPHA)

        for x in range(0, 10):
            for y in range(0, 10):
                if (x + y) % 2 == 0:
                    location = (x * 2 + 1, y * 2 + 1)
                    size = (2, 2)
                    DrawHelper.input_box(self.size, graphics, location, size, self.temp_text_color)

        self.window.blit(graphics, (0, 0))
        pygame.display.update()

    def button_list_func(self, insert):
        self.button_list.append(insert)

    def addbackground(self, graphics):
        background_color = (61, 120, 180)
        background = pygame.Surface(self.size)
        background.fill(background_color)
        graphics.blit(background, (0, 0))

    def clear_button_list(self):
        self.button_list = []

    def text_list_func(self, insert):
        self.text_entry_list.append(insert)

    def clear_text_list(self):
        self.text_entry_list = []

    def create_notification(self, text='Wooo.... Error', array=False):
        self.notify_window_up = True

        background_color = (61, 120, 180)
        background = pygame.Surface(self.size)
        background.fill(background_color)
        background.blit(self.graphics, (0, 0))
        self.graphics = background

        popup_size = (400, 300)

        self.notify_windows.append(
            PopUp(
                (self.size[0] // 2 - popup_size[0] // 2,
                 self.size[1] // 2 - popup_size[1] // 2),
                self.manager, text, array, popup_size))

    def ct(self, string):
        if string == 'button':
            return 'button', self.manager, self.button_list_func
        elif string == 'text':
            return 'text', self.manager, self.text_list_func

    def graphics_banner(self, text='Cool fading banner wooooo'):
        graphics = pygame.Surface(self.size, flags=pygame.SRCALPHA)

        height = 100
        width = self.size[0]

        #font = pygame.font.Font(self.font, 64)

        transparency = 255
        transparent_step = 2

        shadow_color = (0, 0, 0, transparency)

        element_color = (255, 255, 255, transparency)

        #draw shadow
        location = (0, 6)
        size = (20, 8)
        DrawHelper.rectangle(graphics, location, size[0], size[1], shadow_color)

        #draw center
        location = (0, 8)
        size = (20, 4)
        DrawHelper.rectangle(graphics, location, size[0], size[1], element_color)

        #animation
        self.animations.append([graphics, transparency // transparent_step, transparency, transparent_step])

    def graphics_main(self):
        graphics = pygame.Surface(self.size, flags=pygame.SRCALPHA)

        self.addbackground(graphics)

        # title name text
        location = (10, 4)  # centered
        DrawHelper.big_text('Frontiersman', self.size, graphics, location, self.prompt_color, self.large_font)

        # join game box
        location = (10, 8)
        size = (8, 2)
        DrawHelper.input_box(self.size, graphics, location, size, self.prompt_color, 0, self.ct('button'))
        # join game text
        location = (10, 8)  # centered
        DrawHelper.big_text('Join', self.size, graphics, location, self.text_color, self.font)
        # create game box
        location = (10, 11)
        size = (8, 2)
        DrawHelper.input_box(self.size, graphics, location, size, self.prompt_color, 0, self.ct('button'))
        # create game text
        location = (10, 11)  # centered
        DrawHelper.big_text('Host', self.size, graphics, location, self.text_color, self.font)
        # exit game box
        location = (10, 14)  # centered
        size = (8, 2)
        DrawHelper.input_box(self.size, graphics, location, size, self.prompt_color, 0, self.ct('button'))
        # exit game text
        location = (10, 14)  # centered
        DrawHelper.big_text('Exit', self.size, graphics, location, self.text_color, self.font)

        self.graphics = graphics
        #pygame.display.update()

    def graphics_join(self):

        graphics = pygame.Surface(self.size, flags=pygame.SRCALPHA)

        self.addbackground(graphics)

        # name text
        location = (10, 3)  # centered
        DrawHelper.big_text('Name', self.size, graphics, location, self.prompt_color, self.font)

        # invite text
        location = (10, 9)  # centered
        DrawHelper.big_text('Invite Code', self.size, graphics, location, self.prompt_color, self.font)

        # Name input box
        location = (10, 5)
        size = (8, 2)
        DrawHelper.input_box(
            self.size, graphics, location, size, self.prompt_color, 0, self.ct('text'), name='#name_input')

        # invite code box
        location = (10, 11)
        size = (8, 2)
        DrawHelper.input_box(
            self.size, graphics, location, size, self.prompt_color, 0, self.ct('text'), name='#invite_code')

        padding = 5

        # join button
        location = (12, 15)
        size = (4, 2)
        DrawHelper.input_box(self.size, graphics, location, size, self.prompt_color, padding, self.ct('button'))

        # join button text
        location = (12, 15)
        DrawHelper.big_text('Join', self.size, graphics, location, self.text_color, self.font)

        # back button
        location = (8, 15)
        size = (4, 2)
        DrawHelper.input_box(self.size, graphics, location, size, self.prompt_color, padding, self.ct('button'))

        # back button text
        location = (8, 15)
        DrawHelper.big_text('Back', self.size, graphics, location, self.text_color, self.font)

        # self.window.blit(graphics, (0, 0))
        self.graphics = graphics
        #pygame.display.update()

    def graphics_create(self):

        graphics = pygame.Surface(self.size, flags=pygame.SRCALPHA)

        self.addbackground(graphics)

        # name text
        location = (10, 5)  # centered
        DrawHelper.big_text('Name', self.size, graphics, location, self.prompt_color, self.font)

        # Name input box
        location = (10, 7)
        size = (8, 2)
        DrawHelper.input_box(
            self.size, graphics, location, size, self.prompt_color, 0, self.ct('text'), name='#name_input')

        padding = 5

        # back button
        location = (8, 12)
        size = (4, 2)
        DrawHelper.input_box(self.size, graphics, location, size, self.prompt_color, padding, self.ct('button'))
        # back button text
        location = (8, 12)
        DrawHelper.big_text('Back', self.size, graphics, location, self.text_color, self.font)

        # join button
        location = (12, 12)
        size = (4, 2)
        DrawHelper.input_box(self.size, graphics, location, size, self.prompt_color, padding, self.ct('button'))
        # join button text
        location = (12, 12)
        DrawHelper.big_text('Join', self.size, graphics, location, self.text_color, self.font)

        # self.window.blit(graphics, (0, 0))
        self.graphics = graphics
        #pygame.display.update()

    def graphics_lobby(self):
        graphics = pygame.Surface(self.size, flags=pygame.SRCALPHA)

        self.addbackground(graphics)

        '''
        #text
        location = (10, 3)  # centered
        DrawHelper.big_text('Name', self.size, graphics, location, self.prompt_color, self.font)

        #rectangle
        location = (10, 9)  # centered
        size = (1, 1)
        DrawHelper.input_box(self.size, graphics, location, size, self.prompt_color)
        '''

        # RECTANGLES

        padding = 5

        # top area
        location = (10, 3)
        size = (12, 2)
        DrawHelper.input_box(self.size, graphics, location, size, self.prompt_color, padding)

        # top text
        location = (10, 3)  # centered
        DrawHelper.big_text(
            'Lobby Code: ' + str(self.lobby_code), self.size, graphics, location, self.text_color, self.font)

        # left area

        # left title area
        location = (6, 5)  # centered
        size = (4, 2)
        DrawHelper.input_box(self.size, graphics, location, size, self.prompt_color, padding)

        # left title
        DrawHelper.big_text('Lobby History', self.size, graphics, location, self.text_color,
                            self.small_font)

        # left text area
        location = (6, 11)  # centered
        size = (4, 10)
        DrawHelper.input_box(self.size, graphics, location, size, self.prompt_color, padding)
        # left text
        padding = 1
        count = 0
        location = (6, 7 - 1)
        for message in reversed(self.lobby_status):
            if count < 9:
                location = (location[0], location[1] + padding)
                DrawHelper.big_text(
                    message, self.size, graphics, location, self.text_color, self.small_font, width=size[0])
                count += 1
            else:
                break
        # right area

        padding = 5

        # right title area
        location = (12, 5)  # centered
        size = (8, 2)
        DrawHelper.input_box(self.size, graphics, location, size, self.prompt_color, padding)
        # right title
        location = (12, 5)
        DrawHelper.big_text('Players:  ' + str(len(self.players)) + '/' + '4',
                            self.size, graphics, location, self.text_color, self.font)
        # right text area
        location = (12, 11)  # centered
        size = (8, 10)
        DrawHelper.input_box(self.size, graphics, location, size, self.prompt_color, padding)
        # right text
        index = 1
        count = 0
        location = (12, 7 - 2)
        for player in self.players:
            if count < 5:
                location = (location[0], location[1] + index * 2)
                DrawHelper.big_text(player.name + ' [' + player.color + ']', self.size, graphics, location, self.text_color, self.font, width=size[0])
                count += 1
            else:
                break
        # middle button
        location = (14, 17)  # centered
        size = (4, 2)
        DrawHelper.input_box(self.size, graphics, location, size, self.prompt_color, padding, self.ct('button'))

        # left button (only if host)
        if self.is_host:
            location = (6, 17)  # centered
            size = (4, 2)
            DrawHelper.input_box(self.size, graphics, location, size, self.prompt_color, padding, self.ct('button'))
        # TEXT

        # left area

        # right area

        # left button (only if host)
        if self.is_host:
            location = (6, 17)  # centered
            DrawHelper.big_text('Start', self.size, graphics, location, self.text_color, self.font)
        # middle button
        location = (14, 17)  # centered
        DrawHelper.big_text('Leave', self.size, graphics, location, self.text_color, self.font)

        # self.window.blit(graphics, (0, 0))
        self.graphics = graphics
        #pygame.display.update()

    def init_pygame(self):
        pygame.init()
        pygame.font.init()
        self.font = pygame.font.Font("./data/fonts/impact.ttf", self.size[1] // 12)
        self.large_font = pygame.font.Font("./data/fonts/impact.ttf", self.size[1] // 6)
        self.small_font = pygame.font.Font("./data/fonts/impact.ttf", self.size[1] // 24)


class DrawHelper:
    @staticmethod
    def x_centered(size, location):
        return location[0] - size[0] // 2, location[1]

    @staticmethod
    def y_centered(size, location):
        return location[0], location[1] - size[1] // 2

    @staticmethod
    def xy_centered(size, location):
        return location[0] - size[0] // 2, location[1] - size[1] // 2

    @staticmethod
    def input_box(screen_size, surface, location, size, color, padding=0, function=None, name=None, radius=10):
        # tired of copy pasting, function is [string (type of function), ui manager, callback for adding it]
        h_scale = screen_size[0] // 20
        v_scale = screen_size[1] // 20
        size = (size[0] * h_scale - padding, size[1] * v_scale - padding)
        location = (location[0] * h_scale, location[1] * v_scale)
        location = DrawHelper.xy_centered(size, location)
        if function is not None:
            if function[0] == 'button':
                function[2](
                    pygame_gui.elements.UIButton(
                        relative_rect=pygame.Rect(location, size),
                        text='', manager=function[1],
                        object_id='#menu_button_rounded')
                )
                DrawHelper.rounded_rectangle(
                    surface, location, size[0],
                    size[1], radius, color)
            if function[0] == 'text':
                text_entry = UITextEntryLine(
                    relative_rect=pygame.Rect(location, size),
                    manager=function[1],
                    object_id=name)
                text_entry.set_forbidden_characters([',', '|', '&', '\\', '#'])
                function[2](text_entry)
                DrawHelper.rounded_rectangle(
                    surface, location, size[0],
                    size[1], radius, (61, 120, 180))
        else:
            DrawHelper.rounded_rectangle(
                surface, location, size[0],
                size[1], radius, color)

    @staticmethod
    def big_text(text, screen_size, surface, location, color, font, width=None):
        text_surface = pygame.font.Font.render(font, text, True, color, None)
        if width is None:
            pass
        else:
            h_scale = screen_size[0] // 20
            max_width = width * h_scale - h_scale * 0.5
            text_surface = pygame.font.Font.render(font, text, True, color, None)
            if text_surface.get_width() >= max_width:
                new_ratio = text_surface.get_width() / max_width
                new_height = int(text_surface.get_height() / new_ratio)
                new_width = int(text_surface.get_width() / new_ratio)
                text_surface = pygame.transform.scale(text_surface, (new_width, new_height))
        position = (int(screen_size[0] // 20 * location[0] - text_surface.get_width() / 2),
                    int(screen_size[1] // 20 * location[1] - text_surface.get_height() / 2))
        surface.blit(text_surface, position)

    @staticmethod
    def left_text(text, screen_size, surface, location, color, font, padding):
        text = pygame.font.Font.render(font, text, True, color, None)
        position = (int(screen_size[0] // 20 * location[0] + padding),
                    int(screen_size[1] // 20 * location[1]) - text.get_height() / 2)
        surface.blit(text, position)

    @staticmethod
    def intify(float_tuple):
        return int(float_tuple[0]), int(float_tuple[1])

    @staticmethod
    def rounded_rectangle(surface, location, width, height, radius, color):
        circle_x = [location[0] + radius, location[0] + width - radius - 1]
        circle_y = [location[1] + radius, location[1] + height - radius - 1]

        circle_centers = [
            (circle_x[0], circle_y[0]),
            (circle_x[0], circle_y[1]),
            (circle_x[1], circle_y[0]),
            (circle_x[1], circle_y[1])
        ]

        for points in circle_centers:
            pygame.gfxdraw.filled_circle(surface, points[0], points[1], radius, color)

        pygame.gfxdraw.box(surface, pygame.Rect(
            DrawHelper.intify((location[0] + radius, location[1])),
            DrawHelper.intify((width - radius * 2, height))
        ), color)

        pygame.gfxdraw.box(surface, pygame.Rect(
            DrawHelper.intify((location[0], location[1] + radius)),
            DrawHelper.intify((width, height - radius * 2))
        ), color)

    @staticmethod
    def rectangle(surface, location, width, height, color):

        h_scale = surface.get_width() // 20
        v_scale = surface.get_height() // 20

        location = DrawHelper.intify((h_scale * location[0], v_scale * location[1]))
        size = DrawHelper.intify((h_scale * width, v_scale * height))

        pygame.gfxdraw.box(surface, pygame.Rect(
            DrawHelper.intify(location),
            DrawHelper.intify(size)
        ), color)


if __name__ == "__main__":
    GameLobby((1200, 900))



import pygame
import pygame_gui as pg_g


class GuiHelper:
    @staticmethod
    def client_gui(spacing, action_size, manager, resolution, card_size, info_width, card_images):
        action_panel_layout = pygame.Rect(0, 0, 6 * action_size[0] + 7 * spacing, action_size[1] + 2 * spacing)
        action_panel_layout.bottomright = (-spacing, -spacing)
        action_panel = pg_g.elements.UIPanel(relative_rect=action_panel_layout,
                                             starting_layer_height=1,
                                             manager=manager,
                                             anchors={'left': 'right',
                                                      'right': 'right',
                                                      'top': 'bottom',
                                                      'bottom': 'bottom'}
                                             )

        trade_button = pg_g.elements.UIButton(
            relative_rect=pygame.Rect((0 * action_size[0] + 0 * spacing, 0), action_size),
            container=action_panel,
            manager=manager,
            text="TRADE")

        road_button = pg_g.elements.UIButton(
            relative_rect=pygame.Rect((1 * action_size[0] + 1 * spacing, 0), action_size),
            container=action_panel,
            manager=manager,
            text="ROAD")

        settlement_button = pg_g.elements.UIButton(
            relative_rect=pygame.Rect((2 * action_size[0] + 2 * spacing, 0), action_size),
            container=action_panel,
            manager=manager,
            text="SETTLE")

        city_button = pg_g.elements.UIButton(
            relative_rect=pygame.Rect((3 * action_size[0] + 3 * spacing, 0), action_size),
            container=action_panel,
            manager=manager,
            text="CITY")

        development_button = pg_g.elements.UIButton(
            relative_rect=pygame.Rect((4 * action_size[0] + 4 * spacing, 0), action_size),
            container=action_panel,
            manager=manager,
            text="DEV CARD")

        end_turn_button = pg_g.elements.UIButton(
            relative_rect=pygame.Rect((5 * action_size[0] + 5 * spacing, 0), action_size),
            container=action_panel,
            manager=manager,
            text="END",
        )

        card_panel_layout = pygame.Rect(0, 0, resolution[0] - action_panel_layout.width - 2 * spacing,
                                        card_size[1] + 2 * spacing)
        print(action_panel_layout.width, '| Spacing: ', spacing, )
        card_panel_layout.bottomleft = (spacing, -spacing)
        card_panel = pg_g.elements.UIPanel(relative_rect=card_panel_layout,
                                           starting_layer_height=1,
                                           manager=manager,
                                           anchors={'left': 'left',
                                                    'right': 'right',
                                                    'top': 'bottom',
                                                    'bottom': 'bottom'}
                                           )

        info_panel_layout = pygame.Rect(0, 0, info_width, resolution[1] - action_panel_layout.height - 2 * spacing)
        info_panel_layout.topright = (-spacing, spacing)
        info_panel = pg_g.elements.UIPanel(relative_rect=info_panel_layout,
                                           starting_layer_height=1,
                                           manager=manager,
                                           anchors={'left': 'right',
                                                    'right': 'right',
                                                    'top': 'top',
                                                    'bottom': 'top'}
                                           )

        bank_panel = pg_g.elements.UIPanel(relative_rect=pygame.Rect(0, 0, info_width - 2 * spacing, 100),
                                           container=info_panel,
                                           starting_layer_height=1,
                                           manager=manager)

        bank_logo = pg_g.elements.UITextBox(relative_rect=pygame.Rect(spacing, spacing, 60, 60),
                                            container=bank_panel,
                                            manager=manager,
                                            html_text="Bank"
                                            )

        wood_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(bank_logo.relative_rect.right + spacing, spacing, 40, 50),
            image_surface=card_images["Wood"],
            manager=manager,
            container=bank_panel)

        wood_bank = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(wood_logo.relative_rect.left, wood_logo.relative_rect.bottom + spacing, 40, 40),
            html_text="20",
            manager=manager,
            container=bank_panel)

        brick_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(wood_logo.relative_rect.right + spacing, spacing, 40, 50),
            image_surface=card_images["Brick"],
            manager=manager,
            container=bank_panel)

        brick_bank = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(brick_logo.relative_rect.left, brick_logo.relative_rect.bottom + spacing, 40, 40),
            html_text="20",
            manager=manager,
            container=bank_panel)

        sheep_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(brick_logo.relative_rect.right + spacing, spacing, 40, 50),
            image_surface=card_images["Sheep"],
            manager=manager,
            container=bank_panel)

        sheep_bank = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(sheep_logo.relative_rect.left, sheep_logo.relative_rect.bottom + spacing, 40, 40),
            html_text="20",
            manager=manager,
            container=bank_panel)

        wheat_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(sheep_logo.relative_rect.right + spacing, spacing, 40, 50),
            image_surface=card_images["Wheat"],
            manager=manager,
            container=bank_panel)

        wheat_bank = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(wheat_logo.relative_rect.left, wheat_logo.relative_rect.bottom + spacing, 40, 40),
            html_text="20",
            manager=manager,
            container=bank_panel)

        ore_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(wheat_logo.relative_rect.right + spacing, spacing, 40, 50),
            image_surface=card_images["Ore"],
            manager=manager,
            container=bank_panel)

        ore_bank = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(ore_logo.relative_rect.left, ore_logo.relative_rect.bottom + spacing, 40, 40),
            html_text="20",
            manager=manager,
            container=bank_panel)

        end_turn_button.disable()
        development_button.disable()
        city_button.disable()
        settlement_button.disable()
        road_button.disable()
        trade_button.disable()

        return end_turn_button, trade_button, info_panel, action_panel, card_panel



import pygame
import pygame_gui as pg_g
from frontiersman.client import ClientHelper

from frontiersman.client.GuiConstants import INFO_WIDTH, RESOLUTION, SPACING, CARD_SIZE

BANK_SECTION_SIZE = int(INFO_WIDTH / 8)
BANK_HEIGHT = BANK_SECTION_SIZE * 2

PLAYER_HEIGHT = int(INFO_WIDTH / 4)
PLAYER_SECTION_SIZE = int((INFO_WIDTH - 3 * SPACING) / 8)

INFO_LOGOS = ClientHelper.TextureScale.scale_logos((PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE))
BANK_IMAGES = ClientHelper.TextureScale.scale_bank(CARD_SIZE)


class PlayerInfo(pg_g.elements.UIPanel):
    def __init__(self, relative_rect, starting_layer_height, manager, anchors=None):
        super().__init__(relative_rect, starting_layer_height, manager, anchors=anchors)
        self.manager = manager

        self.bank_element = pg_g.elements.UIPanel(
            relative_rect=pygame.Rect(0, 0, INFO_WIDTH, BANK_HEIGHT),
            container=self,
            starting_layer_height=1,
            manager=self.manager)

        self.player_elements = []

    def add_player_element(self, index, player):
        curr_player_info = pg_g.elements.UIPanel(
            relative_rect=pygame.Rect(0, (PLAYER_HEIGHT + SPACING) * index + BANK_HEIGHT + SPACING, INFO_WIDTH - 2 * SPACING, PLAYER_HEIGHT + 2 * SPACING),
            container=self,
            starting_layer_height=1,
            manager=self.manager,
            object_id="#" + player['color']
        )
        name = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(0, 0, 2 * PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            html_text=player['name'],
            manager=self.manager,
        )

        vp_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(name.relative_rect.right, name.relative_rect.top, PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            image_surface=INFO_LOGOS['star'],
            manager=self.manager,
        )
        vp_value = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(vp_logo.relative_rect.right, vp_logo.relative_rect.top, PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            html_text=player['vp'],
            manager=self.manager,
        )

        longest_road_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(vp_value.relative_rect.right, vp_value.relative_rect.top, PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            image_surface=INFO_LOGOS['road'],
            manager=self.manager,
        )
        longest_road_value = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(longest_road_logo.relative_rect.right, longest_road_logo.relative_rect.top, PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            html_text=player['lr'],
            manager=self.manager,
        )

        largest_army_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(longest_road_value.relative_rect.right, longest_road_value.relative_rect.top, PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            image_surface=INFO_LOGOS['army'],
            container=curr_player_info,
            manager=self.manager,
        )
        largest_army_value = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(largest_army_logo.relative_rect.right, largest_army_logo.relative_rect.top, PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            html_text=player['la'],
            manager=self.manager,
        )

        hand_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(name.relative_rect.left, name.relative_rect.bottom, PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            image_surface=INFO_LOGOS['cards'],
            manager=self.manager,
        )
        hand_value = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(hand_logo.relative_rect.right, hand_logo.relative_rect.top,
                                      PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            html_text=player['hand'],
            manager=self.manager,
        )

        roads_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(hand_value.relative_rect.right, hand_value.relative_rect.top, PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            image_surface=INFO_LOGOS['num_roads'],
            manager=self.manager,
        )
        roads_value = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(roads_logo.relative_rect.right, roads_logo.relative_rect.top, PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            html_text=player['roads'],
            manager=self.manager,
        )

        settlements_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(roads_value.relative_rect.right, roads_value.relative_rect.top, PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            image_surface=INFO_LOGOS['settlement'],
            manager=self.manager,
        )
        settlements_value = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(settlements_logo.relative_rect.right, roads_value.relative_rect.top, PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            html_text=player['settlements'],
            manager=self.manager,
        )

        cities_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(settlements_value.relative_rect.right, settlements_value.relative_rect.top, PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            image_surface=INFO_LOGOS['city'],
            manager=self.manager,
        )
        cities_value = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(cities_logo.relative_rect.right, cities_logo.relative_rect.top, PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            html_text=player['city'],
            manager=self.manager,
        )

        self.player_elements.append(curr_player_info)

    def set_player_info(self, current_player, enemy_players):
        for player_element in self.player_elements:
            player_element.kill()

        curr_player = {
            'color': current_player.color.capitalize(),
            'name': current_player.name,
            'vp': str(current_player.victoryPoints) + ' (' + str(current_player.hiddenVictoryPoints) + ')',
            'lr': str(current_player.longestRoad),
            'la': str(current_player.largestArmy),
            'hand': str(current_player.resourceHand.totalResources),
            'roads': str(current_player.numRoads),
            'settlements': str(current_player.numSettlements),
            'city': str(current_player.numCities),
        }
        self.add_player_element(0, curr_player)

        for i, enemy in enumerate(enemy_players):
            player = {
                'color': enemy.color.capitalize(),
                'name': enemy.name,
                'vp': str(enemy.visibleVictoryPoints),
                'lr': str(enemy.longestRoad),
                'la': str(enemy.largestArmy),
                'hand': str(enemy.handSize),
                'roads': str(enemy.numRoads),
                'settlements': str(enemy.numSettlements),
                'city': str(enemy.numCities),
            }
            self.add_player_element(i + 1, player)

    def set_bank_info(self, bank):
        self.bank_element.kill()
        self.bank_element = pg_g.elements.UIPanel(
            relative_rect=pygame.Rect(0, 0, INFO_WIDTH, BANK_HEIGHT + 2 * SPACING),
            container=self,
            starting_layer_height=1,
            manager=self.manager)

        bank_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(0, 0, BANK_SECTION_SIZE * 2, BANK_SECTION_SIZE * 2),
            container=self.bank_element,
            manager=self.manager,
            image_surface=INFO_LOGOS['bank'],
        )

        wood_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(bank_logo.relative_rect.right, bank_logo.relative_rect.top, BANK_SECTION_SIZE, BANK_SECTION_SIZE),
            image_surface=BANK_IMAGES["Wood"],
            manager=self.manager,
            container=self.bank_element)

        brick_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(wood_logo.relative_rect.right, wood_logo.relative_rect.top, BANK_SECTION_SIZE, BANK_SECTION_SIZE),
            image_surface=BANK_IMAGES["Brick"],
            manager=self.manager,
            container=self.bank_element)

        sheep_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(brick_logo.relative_rect.right, brick_logo.relative_rect.top, BANK_SECTION_SIZE, BANK_SECTION_SIZE),
            image_surface=BANK_IMAGES["Sheep"],
            manager=self.manager,
            container=self.bank_element)

        wheat_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(sheep_logo.relative_rect.right, sheep_logo.relative_rect.top, BANK_SECTION_SIZE, BANK_SECTION_SIZE),
            image_surface=BANK_IMAGES["Wheat"],
            manager=self.manager,
            container=self.bank_element)

        ore_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(wheat_logo.relative_rect.right, wheat_logo.relative_rect.top, BANK_SECTION_SIZE, BANK_SECTION_SIZE),
            image_surface=BANK_IMAGES["Ore"],
            manager=self.manager,
            container=self.bank_element)

        dev_logo = pg_g.elements.UIImage(  # todo
            relative_rect=pygame.Rect(ore_logo.relative_rect.right, ore_logo.relative_rect.top, BANK_SECTION_SIZE, BANK_SECTION_SIZE),
            image_surface=BANK_IMAGES["Dev"],
            manager=self.manager,
            container=self.bank_element)

        wood_text = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(wood_logo.relative_rect.left, wood_logo.relative_rect.bottom, BANK_SECTION_SIZE, BANK_SECTION_SIZE),
            html_text=str(bank.lumber),
            manager=self.manager,
            container=self.bank_element,
        )

        brick_text = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(brick_logo.relative_rect.left, brick_logo.relative_rect.bottom, BANK_SECTION_SIZE, BANK_SECTION_SIZE),
            html_text=str(bank.brick),
            manager=self.manager,
            container=self.bank_element,
        )

        sheep_text = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(sheep_logo.relative_rect.left, sheep_logo.relative_rect.bottom, BANK_SECTION_SIZE, BANK_SECTION_SIZE),
            html_text=str(bank.wool),
            manager=self.manager,
            container=self.bank_element,
        )

        wheat_text = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(wheat_logo.relative_rect.left, wheat_logo.relative_rect.bottom, BANK_SECTION_SIZE, BANK_SECTION_SIZE),
            html_text=str(bank.grain),
            manager=self.manager,
            container=self.bank_element,
        )

        ore_text = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(ore_logo.relative_rect.left, ore_logo.relative_rect.bottom, BANK_SECTION_SIZE, BANK_SECTION_SIZE),
            html_text=str(bank.ore),
            manager=self.manager,
            container=self.bank_element,
        )

        dev_text = pg_g.elements.UITextBox(
            relative_rect=pygame.Rect(dev_logo.relative_rect.left, dev_logo.relative_rect.bottom, BANK_SECTION_SIZE, BANK_SECTION_SIZE),
            html_text=str(bank.totalDevelopment),
            manager=self.manager,
            container=self.bank_element,
        )



class Tile:
    def __init__(self, resource='Wheat', location=[0, 0, 0]):
        self.__resource = resource
        self.__robber = False
        self.__location = location
        self.__roads = []
        self.__buildings = []
        self.__number = 0
        # roads and building list start due north and iterate clockwise

    def get_number(self):
        return self.__number

    def set_number(self, integer):
        self.__number = integer

    def get_resource(self):
        return self.__resource

    def set_robber(self, value):
        self.__robber = value

    def update_buildings(self, index, player):
        self.__buildings[index]

    def get_robber(self):
        return self.__robber

    def get_buildings(self):
        return self.__buildings

    def get_location(self):
        return self.__location

    def set_location(self, location):
        self.__location = location

    robber = property(get_robber, set_robber)
    buildings = property(get_buildings, update_buildings)
    resource = property(get_resource)
    location = property(get_location, set_location)
    number = property(get_number, set_number)



import sys


def build_road_check(actor, free=False):
    hand = actor.resourceHand
    if free and actor.numRoads > 0:
        return True
    if hand.brick < 1 or hand.lumber < 1 or actor.numRoads == 0:
        return False
    else:
        return True


def build_road_available(actor, free=False):
    to_return = list()
    if build_road_check(actor, free):
        for edgeBase in actor.ownedRoads:
            for corner in edgeBase.corners:
                for edge in corner.edges:
                    if edge is not None:
                        if edge.available and edge not in to_return:
                            to_return.append(edge)
        return to_return
    else:
        return to_return


def build_settlement_check(actor):
    hand = actor.resourceHand
    if hand.brick < 1 or hand.lumber < 1 or hand.grain < 1 or hand.wool < 1 or actor.numSettlements == 0:
        return False
    else:
        return True


def build_settlement_available(actor):
    toReturn = list()
    if build_settlement_check(actor):
        for edgeBase in actor.ownedRoads:
            for corner in edgeBase.corners:
                if corner.available and corner not in toReturn:
                    toReturn.append(corner)
        return toReturn
    else:
        return toReturn


def build_city_check(actor):
    hand = actor.resourceHand
    if hand.grain < 2 or hand.ore < 3:  # or actor.numCities == 0 or actor.numSettlements == 5:
        return False
    else:
        return True


def build_city_available(actor):
    toReturn = list()
    if build_city_check(actor):
        for corner in actor.ownedNodes:
            if not corner.city:
                toReturn.append(corner)
        return toReturn
    else:
        return toReturn


def buy_development_check(actor):
    hand = actor.resourceHand
    if hand.grain < 1 or hand.ore < 1 or hand.wool < 1:
        return False
    else:
        return True


# todo not used

def buy_development(actor, bank):
    if buy_development_check(actor):
        '''
        randomCard = random.uniform(0, bank.totalDevelopments)

        if randomCard - bank.knights < 0:
            bank.knights -= 1
            actor.developmentHand.knights += 1
        else:
            randomCard -= bank.knights

        if randomCard - bank.roadBuildings < 0:
            bank.roadBuildings -= 1
            actor.developmentHand.roadBuildings += 1
        else:
            randomCard -= bank.roadBuildings

        if randomCard - bank.yearOfPlenty < 0:
            bank.yearOfPlenty -= 1
            actor.developmentHand.yearOfPlenty += 1
        else:
            randomCard -= bank.yearOfPlenty

        if randomCard - bank.monopolies < 0:
            bank.monopolies -= 1
            actor.developmentHand.monopolies += 1
        else:
            randomCard -= bank.monopolies

        if randomCard - bank.victoryPoints < 0:
            bank.victoryPoints -= 1
            actor.developmentHand.victoryPoints += 1
        else:
            randomCard -= bank.victoryPoints
        '''
        print("check passed")

    else:
        return None


# todo not used
def trade(offeredCards, requestedCards, playerResources, bankResources):
    pass
